#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from __future__ import with_statement
from cocktail.language import get_content_language, content_language_context
from cocktail.modeling import getter, ListWrapper
from cocktail.translations import translations
from cocktail.schema import (
    Member, Boolean, Reference, BaseDateTime, Decimal, Collection
)
from cocktail.controllers.fileupload import FileUpload
from cocktail.html import Element
from cocktail.html.datadisplay import DataDisplay
from cocktail.html.hiddeninput import HiddenInput

# Extension property that allows members to define their appearence in HTML
# forms
Member.edit_control = None


class Form(Element, DataDisplay):
    """A class that generates HTML forms from a schema description.

    @var errors: A list of validation errors, used to highlight fields with
        incorrect values.
    @type errors: L{ErrorList<cocktail.schema.errorlist.ErrorList>}

    @var embeded: When set to True, the form turns into a fieldset, to be
        nested inside other forms.
    @type embeded: bool

    @var required_marks: Sets wether a small visual clue should be added next
        to required fields.
    @type required_marks: bool

    @var table_layout: Determines if the form should use a table to align its
        fields.
    @type table_layout: bool

    @var default_button: A reference to one of the form's buttons. When set,
        the referenced button is duplicated, hidden and inserted at the start
        of the form, which makes that button act as the default for the whole
        form. This can be very useful when dealing with multiple buttons on a
        single form. Doesn't apply to L{embeded} forms.
    @type default_button: L{Element<cocktail.html.element.Element>}
    
    @var generate_fields: Indicates if the form should automatically create
        entries for all fields defined by its assigned schema. This is the
        default behavior; When set to False, filling the form will be left to
        the client code.
    @type generate_fields: bool
    """

    tag = "form"
    translations = None
    hide_empty_fieldsets = True
    errors = None
    hidden = False
    embeded = False
    required_marks = True
    table_layout = False
    default_button = None
    generate_fields = True

    def __init__(self, *args, **kwargs):
        DataDisplay.__init__(self)

        self.set_member_type_display(
            Boolean, "cocktail.html.CheckBox")

        self.set_member_type_display(
            Reference, "cocktail.html.DropdownSelector")
            
        self.set_member_type_display(
            BaseDateTime, "cocktail.html.DatePicker")

        self.set_member_type_display(
            Decimal, "cocktail.html.DecimalBox")

        self.set_member_type_display(
            FileUpload, "cocktail.html.FileUploadBox")

        self.set_member_type_display(
            Collection, "cocktail.html.CheckList")

        self.__groups = []
        self.groups = ListWrapper(self.__groups)
        self.__hidden_members = {}
        Element.__init__(self, *args, **kwargs)

    def _build(self):
        
        self.add_resource("/cocktail/scripts/form.js")

        self.fields = Element()
        self.fields.add_class("fields")
        self.append(self.fields)

        self.buttons = Element()
        self.buttons.add_class("buttons")
        self.append(self.buttons)

        self.submit_button = Element("button")
        self.submit_button["value"] = "true"
        self.submit_button["type"] = "submit"
        self.submit_button.append(translations("Submit"))
        self.buttons.append(self.submit_button)
        self.default_button = self.submit_button

    def _ready(self):
        self._fill_fields()

        if self.embeded:
            if self.tag == "form":
                self.tag = "fieldset"
            self.buttons.visible = False

        elif self.default_button:
            hidden_button = Element(self.default_button.tag)
            for key, value in hidden_button.attributes.itervalues():
                hidden_button[key] = value
            hidden_button.set_style("display", "none")
            self.insert(0, hidden_button)

    def _fill_fields(self):
        if self.schema and self.generate_fields:

            if not self.__groups:
                schema_groups = self.displayed_members_by_group
                if len(schema_groups) > 1:
                    for group_name, members in schema_groups:
                        self.add_group(
                            group_name or "default",
                            set(member.name for member in members)
                        )

            if self.__groups:                
                members = self.displayed_members

                for group in self.__groups:

                    fieldset = self.create_fieldset(group)
                    self.fields.append(fieldset)
                    setattr(self, group.id + "_fieldset", fieldset)

                    if self.table_layout:
                        container = Element("table")
                        fieldset.append(container)
                    else:
                        container = fieldset

                    has_match = False
                    remaining_members = []

                    for member in members:
                        if group.matches(member):
                            field_entry = self.create_field(member)
                            container.append(field_entry)
                            self.build_member_explanation(member, field_entry)
                            setattr(self, member.name + "_field", field_entry)
                            has_match = True
                        else:
                            remaining_members.append(member)

                    members = remaining_members

                    if self.hide_empty_fieldsets and not has_match:
                        fieldset.visible = False
            else:
                if self.table_layout:
                    self.fields.tag = "table"

                for member in self.displayed_members:
                    field_entry = self.create_field(member)
                    self.fields.append(field_entry)
                    self.build_member_explanation(member, field_entry)
                    setattr(self, member.name + "_field", field_entry)
    
    def create_fieldset(self, group):

        group_id = group.id

        fieldset = Element("fieldset") 
        fieldset.add_class(group_id)

        label = self.get_group_label(group_id)
        if label:            
            fieldset.legend = Element("legend")
            fieldset.legend.append(label)
            fieldset.append(fieldset.legend)
        else:
            fieldset.legend = None
            fieldset.add_class("anonymous")

        return fieldset

    def create_field(self, member):

        hidden = self.get_member_hidden(member)
        
        entry = Element("tr" if self.table_layout else "div")

        if hidden:
            entry.tag = None
        else:
            entry.add_class("field")
            entry.add_class(member.name + "_field")
             
            if member.required:
                entry.add_class("required")

        def create_instance():
            if hidden:
                return self.create_hidden_input(self.data, member)
            else:
                return self.create_field_instance(member)

        if member.translated:
            entry.add_class("translated")
            for language in (self.translations or (get_content_language(),)):
                with content_language_context(language):
                    field_instance = create_instance()
                    field_instance.add_class(language)
                    entry.append(field_instance)
        else:
            entry.append(create_instance())
        
        return entry

    def build_member_explanation(self, member, entry):
        explanation = member.get_member_explanation()
        if explanation:
            entry.explanation = \
                self.create_member_explanation(member, explanation)

            if self.table_layout:
                entry.explanation.tag = "td"
                row = Element("tr")
                row.add_class("explanation_row")
                row.append(entry.explanation)
                entry.parent.append(row)
            else:
                entry.append(entry.explanation)

    def create_member_explanation(self, member, explanation):
        label = Element()
        label.add_class("explanation")
        label.append(explanation)
        return label

    def create_field_instance(self, member):

        field_instance = Element("td" if self.table_layout else "div")
        field_instance.add_class("field_instance")
    
        # Label
        field_instance.label = self.create_label(member)
        field_instance.append(field_instance.label)
        
        # Control
        field_instance.control = self.create_control(self.data, member)

        if field_instance.control.class_css:
            for class_name in field_instance.control.class_css.split(" "):
                field_instance.add_class("field_instance-" + class_name)

        insert = getattr(field_instance.control, "insert_into_form", None)

        if insert:
            insert(self, field_instance)
        else:
            field_instance.append(field_instance.control)

        field_instance.label["for"] = field_instance.control.require_id()

        return field_instance

    def create_hidden_input(self, obj, member):

        input = HiddenInput()
        input.data = obj
        input.member = member
        input.value = self.get_member_value(obj, member)

        if member.translated:
            input.language = get_content_language()

        return input

    def create_label(self, member):
        
        label = Element("label")
        text = self.get_member_label(member)
        
        if text:
            label.label_title = self.create_label_title(member, text)
            label.append(label.label_title)
            
            if member.translated:
                
                label.label_language = self.create_language_label(
                    member,
                    get_content_language())

                label.append(label.label_language)

            if self.required_marks \
            and member.required \
            and not isinstance(member, Collection):
                label.required_mark = self.create_required_mark(member)
                label.append(label.required_mark)
        else:
            label.visible = False

        return label

    def create_label_title(self, member, text):
        label_title = Element("span")
        label_title.add_class("label_title")
        label_title.append(text)
        return label_title

    def create_language_label(self, member, language):
        label = Element("span")
        label.add_class("language")
        label.append("(" + translations(language) + ")")
        return label

    def create_required_mark(self, member):
        mark = Element("span")
        mark.add_class("required_mark")
        mark.append("*")
        return mark

    def get_member_supplied_display(self, obj, member):
        return member.edit_control

    def default_display(self, obj, member):
        if member.enumeration is not None:
            return "cocktail.html.DropdownSelector"
        else:
            return "cocktail.html.TextBox"

    def create_control(self, obj, member):
        control = self.get_member_display(obj, member)
        control.add_class("control")

        if self.errors and self.errors.in_member(
            member,
            member.translated and get_content_language() or None
        ):
            control.add_class("error")

        return control   

    def add_group(self, group_id, members_filter):
        self.__groups.append(FormGroup(self, group_id, members_filter))

    def get_member_hidden(self, member):
        key = self._normalize_member(member)
        return self.__hidden_members.get(key, self.hidden)

    def set_member_hidden(self, member, hidden):
        key = self._normalize_member(member)
        self.__hidden_members[key] = hidden


class FormGroup(object):

    def __init__(self, form, id, members_filter):
        self.__form = form
        self.__id = id
        self.__members_filter = members_filter

        if callable(members_filter):
            self.__match_expr = members_filter
        else:
            members = set(self.__form._normalize_member(member)
                          for member in members_filter)

            self.__match_expr = \
                lambda member: self.__form._normalize_member(member) in members

    @getter
    def form(self):
        return self.__form

    @getter
    def id(self):
        return self.__id

    @getter
    def members_filter(self):
        return self.__members_filter

    def matches(self, member):
        return self.__match_expr(member)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2008
"""
from inspect import getmro
from types import FunctionType
from cocktail.modeling import empty_list, extend, call_base
from cocktail.pkgutils import get_full_name

_overlays = {}

def apply_overlays(element):
    """Initialize an element with the modifications provided by all matching
    overlays. Overlays are applied in the same order they were registered in.

    @param element: The element to apply the overlays to.
    @type element: L{Element<cocktail.html.element.Element>}
    """
    if _overlays:
        for cls in element.__class__._classes:
            class_overlays = _overlays.get(get_full_name(cls))
            if class_overlays:
                for overlay in class_overlays:
                    overlay.modify(element)


# Declaration stub, required by the metaclass
Overlay = None


class Overlay(object):
    """Base (abstract) class for all overlays. An overlay represents a set of
    modifications to an L{Element<cocktail.html.element.Element>} class.
    Overlays can extend an existing class with new content and/or alter its
    existing elements, in a clean, reusable and unobtrusive way.
    
    To create an overlay, subclass this base class or another overlay class,
    and invoke the L{register} method on the new class. All methods and
    attributes defined by the overlay will be made available to its target.
    """
    excluded_keys = set(["__module__", "_classes", "__doc__"])

    class __metaclass__(type):
        def __init__(cls, name, bases, members):
            type.__init__(cls, name, bases, members)
            cls._classes = [c
                            for c in reversed(getmro(cls))
                            if Overlay
                            and c is not Overlay
                            and issubclass(c, Overlay)]

    @classmethod
    def register(cls, target):
        """Installs the overlay on the indicated class. All new instances of
        the modified class will reflect the changes introduced by the overlay
        (existing instances will remain unchanged).

        @param target: The class to install the overlay on.
        @type target: Class or str
        """
        if isinstance(target, basestring):
            from cocktail.html import templates
            target = templates.get_class(target)
        
        target_full_name = get_full_name(target)
        
        class_overlays = _overlays.get(target_full_name)

        if not class_overlays:
            class_overlays = []
            _overlays[target_full_name] = class_overlays
     
        class_overlays.append(cls)
       
    @classmethod
    def modify(cls, element):

        for key, value in cls.__dict__.iteritems():
            
            if key in cls.excluded_keys:
                continue

            if isinstance(value, FunctionType):
                extend(element)(value)
            else:
                setattr(element, key, value)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from inspect import getmro

_undefined = object()

class TypeMapping(dict):

    def __getitem__(self, cls):
        
        value = self.get(cls, _undefined)

        if value is _undefined:
            raise KeyError(cls)
        
        return value

    def get(self, cls, default = None):
        for cls in getmro(cls):
            value = dict.get(self, cls, _undefined)
            if value is not _undefined:
                return value
        else:
            return default





#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2007
"""
from cocktail.html import templates
from cocktail.html.element import Element
from cocktail.html.selectable import selectable
from cocktail.html.datadisplay import (
    CollectionDisplay,
    NO_SELECTION, SINGLE_SELECTION, MULTIPLE_SELECTION
)
from cocktail.translations import translations
from cocktail.language import get_content_language, set_content_language
from cocktail.schema import Collection, get
from cocktail.schema.expressions import (
    TranslationExpression,
    PositiveExpression,
    NegativeExpression
)
from cocktail.controllers.viewstate import view_state


class Table(Element, CollectionDisplay):
    
    tag = "table"
    sortable = False
    ascending_order_image = "ascending.png"
    descending_order_image = "descending.png"
    base_image_url = None
    selection_parameter = "selection"
    nested_list_max_length = 5
    translated_values = True
    persistence_prefix = None
    entry_selector = "tbody tr"
    checkbox_selector = "td.selection input"
    resizable_rows_selector = "tbody tr"

    def __init__(self, *args, **kwargs):
        Element.__init__(self, *args, **kwargs)
        CollectionDisplay.__init__(self)
        self.__column_display = {}
        self.__column_labels = {}
        self.set_member_type_display(Collection, self.display_collection)
        self.add_class("resizable")

    def _build(self):

        self.head = Element("thead")
        self.append(self.head)

        self.head_row = Element("tr")
        self.head.append(self.head_row)

        self.body = Element("body")
        self.append(self.body)

    def _ready(self):
                
        Element._ready(self)
        self.add_resource("/cocktail/scripts/jquery.cookie.js")
        self.add_resource("/cocktail/scripts/jquery.tableresizer.js")

        selectable(
            self,
            mode = self.selection_mode,
            entry_selector = self.entry_selector,
            checkbox_selector = self.checkbox_selector
        )

        self.set_client_param(
            "resizableRowsSelector",
            self.resizable_rows_selector
        )

        self.set_client_param("persistencePrefix", self.persistence_prefix)

        if self.grouping:
            self.set_member_displayed(self.grouping.member, False)
            self._grouping_member_translation = \
                u"(" + translations(self.grouping.member) + u")"
            self._remove_grouping_translation = \
                translations("sitebasis.views.ContentTable remove grouping")

        self._fill_head()
        self._fill_body()

    def _fill_head(self):

        # Cache sorted columns
        if self.order:
            
            self._sorted_columns = sorted_columns = {}
            
            for criteria in self.order:
                sign = criteria.__class__
                expr = criteria.operands[0]
                
                if isinstance(expr, TranslationExpression):
                    member = expr.operands[0].name
                    language = expr.operands[1].value
                else:
                    member = expr.name
                    language = None

                sorted_columns[(member, language)] = sign

        # Selection column
        if self.selection_mode != NO_SELECTION:
            selection_header = Element("th")
            selection_header.add_class("selection")
            self.head_row.append(selection_header)
        
        # Regular columns
        for column in self.displayed_members:
            if column.translated:
                for language in self.translations:
                    header = self.create_header(column, language)
                    self.head_row.append(header)
            else:
                header = self.create_header(column)
                self.head_row.append(header)
    
    def _fill_body(self):

        if self.grouping:
            undefined = object()
            current_group = undefined        
            get_group = self.grouping.get_grouping_value
        else:
            get_group = None

        for i, item in enumerate(self.data):

            if get_group:
                group = get_group(item)
                if group != current_group:
                    group_row = self.create_group_row(group)
                    self.append(group_row)
                    current_group = group

            row = self.create_row(i, item)
            self.append(row)
            self._row_added(i, item, row)

    def _row_added(self, index, item, row):
        pass

    def create_group_row(self, group):
        
        row = Element("tr")
        row.add_class("group")
        
        cell = Element("td", colspan = "0", children = [
            Element("span",
                class_name = "grouping_value",
                children = [
                    self.grouping.translate_grouping_value(group)
                    or self.translate_value(
                        self.data,
                        self.grouping.member,
                        group
                    )
                ]
            ),
            Element("span",
                class_name = "grouping_member",
                children = [self._grouping_member_translation]
            ),
            Element("a",
                href = u"?" + view_state(grouping = "", page = 0),
                class_name = "remove_grouping",
                children = [self._remove_grouping_translation]
            )
        ])
        row.append(cell)
        
        return row

    def create_row(self, index, item):
        row = Element("tr")
        row.add_class(index % 2 == 0 and "odd" or "even")
        
        if self.selection_mode != NO_SELECTION:
            row.append(self.create_selection_cell(item))

        if self.schema.primary_member:
            row["id"] = item.id
                    
        for column in self.displayed_members:
            if self.translations and column.translated:
                current_content_language = get_content_language()
                for language in self.translations:
                    set_content_language(language)
                    cell = self.create_cell(item, column, language)
                    row.append(cell)
                set_content_language(current_content_language)
            else:
                cell = self.create_cell(item, column)
                row.append(cell)
        
        return row

    def get_item_id(self, item):
        pm = self.schema.primary_member
        if pm is None:
            raise ValueError(
                "Selectable tables must have a schema with a primary member "
                "defined or override their get_item_id() method"
            )
        return get(item, pm)

    def create_selection_cell(self, item):        
        selection_cell = Element("td")
        selection_cell.add_class("selection")
        selection_cell.append(self.create_selection_control(item))
        return selection_cell

    def create_selection_control(self, item):

        id = self.get_item_id(item)

        selection_control = Element("input")
        selection_control["name"] = self.selection_parameter
        selection_control["id"] = "selection_" + str(id)
        selection_control["value"] = id
        selection_control["autocomplete"] = "off"

        if self.selection_mode == SINGLE_SELECTION:
            selection_control["type"] = "radio"
        else:
            selection_control["type"] = "checkbox"

        selection_control["checked"] = self.is_selected(item)

        return selection_control

    def create_header(self, column, language = None):
        
        header = Element("th")
        self._init_cell(header, column, language)
        
        header.label = Element("span")
        header.label.add_class("label")
        header.label.append(self.get_member_label(column))
        header.append(header.label)
        
        # Translation label
        if language:
            header.translation_label = self.create_translation_label(language)
            header.append(header.translation_label)
        
        self.add_header_ui(header, column, language)
        return header       

    def create_translation_label(self, language):
        label = Element("span")
        label.add_class("translation")
        label.append(u"(" + translations(language) + u")")
        return label

    def add_header_ui(self, header, column, language):

        # Sorting
        if self.get_member_sortable(column):
            header.label.tag = "a"
            sign = ""

            if self.order:
                current_direction = self._sorted_columns.get(
                    (column.name, language)
                )

                if current_direction is not None:
                    header.add_class("sorted")

                    if current_direction is PositiveExpression:
                        header.add_class("ascending")
                        sign = "-"
                    elif current_direction is NegativeExpression:
                        header.add_class("descending")
 
            order_param = sign + column.name

            if language:
                order_param += "." + language

            header.label["href"] = "?" + view_state(
                order = order_param,
                page = 0
            )

    def create_cell(self, item, column, language = None):
        cell = Element("td")

        if self.order \
        and (column.name, language) in self._sorted_columns:
            cell.add_class("sorted")

        self._init_cell(cell, column, language)
        display = self.get_member_display(item, column)
        cell.append(display)
        return cell

    def _init_cell(self, cell, column, language = None):
        cell.add_class(column.name + "_column")

        if language:
            cell.add_class(language)

    def display_collection(self, obj, member):
        list = templates.new("cocktail.html.List")
        list.items = self.get_member_value(obj, member)
        list.max_length = self.nested_list_max_length
        return list




#-*- coding: utf-8 -*-
u"""
Provides the base class for all schema members.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			March 2008
"""
from itertools import chain
from copy import deepcopy
from cocktail.events import Event, EventHub
from cocktail.modeling import ListWrapper, OrderedSet
from cocktail.pkgutils import import_object
from cocktail.translations import translations
from cocktail.schema import exceptions
from cocktail.schema.expressions import Expression, Variable
from cocktail.schema.validationcontext import ValidationContext
from cocktail.schema.accessors import get_accessor

class DynamicDefault(object):

    def __init__(self, factory):
        self.factory = factory

    def __call__(self):
        return self.factory()


class Member(Variable):
    """Schema members are the distinct data units that comprise a
    L{schema<schema.Schema>}.

    Members are bound to a single schema, where they are identified by a unique
    name. The purpose of a member is to describe the nature and constraints of
    a discrete piece of data for the schema they belong to. Typical examples of
    members are fields and collections, and their respective subtypes.
    
    This class acts mostly as an abstract type, used as a base by all the
    different kinds of members that can comprise a schema.

    @ivar default: The default value for the member.
    
    @ivar required: Determines if the field requires a value. When set to true,
        a value of None for this member will trigger a validation error of type
        L{ValueRequiredError<exceptions.ValueRequiredError>}.
    @type required: bool
    
    @ivar require_none: Determines if the field disallows any value other than
        None.  When set to true, a value different than None for this member
        will trigger a validation error of type
        L{NoneRequiredError<exceptions.NoneRequiredError>}.
    @type require_none: bool

    @ivar enumeration: Establishes a limited set of acceptable values for the
        member. If a member with this constraint is given a value not found
        inside the set, an L{EnumerationError<exceptions.EnumerationError>}
        error will be triggered.
    @type enumeration: any container
    """
    __metaclass__ = EventHub

    attached = Event(
        doc = """An event triggered when the member is added to a schema."""
    )

    # Groupping
    member_group = None

    # Constraints
    primary = False
    default = None
    required = False
    require_none = False
    enumeration = None

    # Instance data layout
    accessor = None

    # Copying and adaptation
    _copy_class = None
    copy_source = None
    adaptation_source = None
    original_member = None

    # Translation
    translated = False
    translation = None
    translation_source = None
    
    # Attributes that deserve special treatment when performing a deep copy
    _special_copy_keys = set([
        "__class__", "_schema", "_validations_wrapper", "_validations"
    ])

    def __init__(self, name = None, doc = None, **kwargs):
        self._name = None
        self._schema = None
        self._validations = OrderedSet()
        self._validations_wrapper = ListWrapper(self._validations)
        self.add_validation(Member.member_validation_rule)
        self.original_member = self

        Variable.__init__(self, None)
        self.__type = None

        self.name = name
        self.__doc__ = doc

        for key, value in kwargs.iteritems():
            setattr(self, key, value)

    def __repr__(self):
        
        member_desc = self._name \
            and "member '%s'" % self._name \
            or "anonymous " + self.__class__.__name__.lower()

        if self._schema is None:
            return "unbound " + member_desc
        else:
            schema_name = self._schema.name
            return "%s in %s" % (
                member_desc,
                schema_name
                    and "'%s' schema" % schema_name or "anonymous schema"
            )

    def _get_name(self):
        return self._name

    def _set_name(self, value):

        if self._schema is not None:
            raise exceptions.MemberRenamedError(self, value)
        
        self._name = value

    name = property(_get_name, _set_name, doc = """
        The name that uniquely identifies the member on the schema it is bound
        to. Once set it can't be changed (trying to do so will raise a
        L{MemberRenamedError} exception).
        @type: str
        """)

    def _get_schema(self):
        return self._schema

    def _set_schema(self, value):

        if self._schema is not None:
            raise exceptions.MemberReacquiredError(self, value)

        self._schema = value

    schema = property(_get_schema, _set_schema, doc = """
        The schema that the member is bound to. Once set it can't be changed
        (trying to do so will raise a L{MemberReacquiredError} exception).
        @type: L{Schema<schema.Schema>}
        """)

    def _get_type(self):
        
        # Resolve string references
        if isinstance(self.__type, basestring):
            self.__type = import_object(self.__type)
        
        return self.__type

    def _set_type(self, type):
        self.__type = type

    type = property(_get_type, _set_type, doc = """
        Imposes a data type constraint on the member. All values assigned to
        this member must be instances of the specified data type. Breaking this
        restriction will produce a validation error of type
        L{TypeCheckError<exceptions.TypeCheckError>}.
        @type type: type or str
        """)

    def _set_exclusive(self, expr):
        self.required = expr
        
        if isinstance(expr, Expression):        
            self.require_none = expr.not_()
        else:
            self.require_none = lambda ctx: not expr(ctx)

    exclusive = property(None, _set_exclusive, doc = """
        A write only property that eases the definition of members that should
        be required or empty depending on a condition and its negation,
        respectively.
        @type: L{Expression<cocktail.schema.expressions.Expression>}
        """)

    def produce_default(self, instance = None):
        """Generates a default value for the member. Can be overridden (ie. to
        produce dynamic default values).

        @param instance: The instance that the default is produced for.
        @type instance: object

        @return: The resulting default value.
        @rtype: object
        """
        if instance is not None and self.name:
            default = getattr(instance, "default_" + self.name, self.default)
        else:
            default = self.default

        if isinstance(default, DynamicDefault):
            return default()
        else:
            return default

    def copy(self):
        """Creates a deep, unbound copy of the member.

        @return: The resulting copy.
        @rtype: L{Member}
        """
        return deepcopy(self)
    
    def __deepcopy__(self, memo):

        if self._copy_class:
            copy = self._copy_class()
        else:
            copy = self.__class__()

        memo[id(self)] = copy

        for key, value in self.__dict__.iteritems():
            if key not in self._special_copy_keys:
                copy.__dict__[key] = deepcopy(value, memo)
        
        copy._validations = list(self._validations)
        memo[id(self._validations)] = copy._validations
        
        copy._validations_wrapper = ListWrapper(copy._validations)
        memo[id(copy._validations_wrapper)] = copy._validations_wrapper

        copy.copy_source = self

        return copy

    def add_validation(self, validation):
        """Adds a validation function to the member.
        
        @param validation: A callable that will be added as a validation rule
            for the member. Takes two positional parameters (a reference to the
            member itself, and the value assigned to the member), plus any
            additional number of keyword arguments used to refine validation
            options and context. The callable should produce a sequence of
            L{ValidationError<exceptions.ValidatitonError>} instances.
        @type validation: callable

        @return: The validation rule, as provided.
        @rtype: callable
        """
        self._validations.append(validation)
        return validation
    
    def remove_validation(self, validation):
        """Removes one of the validation rules previously added to a member.

        @param validation: The validation to remove, as previously passed to
            L{add_validation}.
        @type validation: callable

        @raise ValueError: Raised if the member doesn't have the indicated
            validation.
        """
        self._validations.remove(validation)

    def validations(self, recursive = True):
        """Iterates over all the validation rules that apply to the member.

        @param recursive: Indicates if the produced set of validations should
            include those declared on members contained within the member. This
            parameter is only meaningful on L{compound members<>}, but is made
            available globally in order to allow the method to be called
            polymorphically using a consistent signature.

        @return: The sequence of validation rules for the member.
        @rtype: callable iterable
        """
        return self._validations_wrapper

    def validate(self, value, context = None, **context_params):
        """Indicates if the given value fulfills all the validation rules
        imposed by the member.
        
        @param value: The value to validate.
        
        @param context: Additional parameters used to fine tune the validation
            process.
        @type context: L{ValidationContext<validationcontext.ValidationContext>}

        @param context_params: Arbitrary keyword parameters to feed the
            validation context with.
        """
        for error in self.get_errors(value, context, **context_params):
            return False

        return True
 
    def get_errors(self, value, context = None, **context_params):
        """Tests the given value with all the validation rules declared by the
        member, iterating over the resulting set of errors.

        @param value: The value to evaluate.
        @param context: Additional parameters used to fine tune the validation
            process.
        @type context: L{ValidationContext<validationcontext.ValidationContext>}

        @param context_params: Arbitrary keyword parameters to feed the
            validation context with.

        @return: An iterable sequence of validation errors.
        @rtype: L{ValidationError<exceptions.ValidationError>}
            iterable
        """
        if context is None:
            context = ValidationContext(self, value, **context_params)

        for validation in self.validations():
            for error in validation(self, value, context):
                yield error

    @classmethod
    def resolve_constraint(cls, expr, context):
        """Resolves a constraint expression for the given context.
        
        Most constraints can be specified using dynamic expressions instead of
        static values, allowing them to adapt to different validation contexts.
        For example, a field may state that it should be required only if
        another field is set to a certain value. This method normalizes any
        constraint (either static or dynamic) to a static value, given a
        certain validation context.

        Dynamic expressions are formed by assigning a callable object or an
        L{Expression<cocktail.schema.expressions.Expression>} instance to a
        constraint value.

        @param expr: The constraint expression to resolve.
        
        @param context: The validation context that will be made available to
            dynamic constraint expressions.
        @type context: L{ValidationContext<validationcontext.ValidationContext>}

        @return: The normalized expression value.
        """
        if not isinstance(expr, type):
            if isinstance(expr, Expression):
                validable = context.validable
                return expr.eval(validable, get_accessor(validable))
            elif callable(expr):
                return expr(context)
        
        return expr

    def member_validation_rule(self, value, context):
        """
        The base validation rule for all members. Tests the L{required},
        L{require_none}, L{enumeration} and L{type} constraints.
        """
        # Value required
        if value is None:
            if self.resolve_constraint(self.required, context):
                yield exceptions.ValueRequiredError(self, value, context)

        # None required
        elif self.resolve_constraint(self.require_none, context):
            yield exceptions.NoneRequiredError(self, value, context)

        else:
            # Enumeration
            enumeration = self.resolve_constraint(self.enumeration, context)
            
            if enumeration is not None and value not in enumeration:
                yield exceptions.EnumerationError(
                    self, value, context, enumeration)

            # Type check
            type = self.resolve_constraint(self.type, context)

            if type and not isinstance(value, type):
                yield exceptions.TypeCheckError(self, value, context, type)

    def __translate__(self, language, **kwargs):        
        return translations(
            self.schema.name + "." + self.name,
            language,
            chain = self.copy_source,
            **kwargs
        )

    def translate_value(self, value, language = None, **kwargs):
        if value is None:
            return u""
        else:
            return unicode(value)

    def get_member_explanation(self, language = None, **kwargs):
        
        explanation = None

        if self.schema and self.schema.name:        
            explanation = translations(
                self.schema.name + "." + self.name + "-explanation",
                language,
                **kwargs
            )

        if not explanation and self.copy_source:
            explanation = self.copy_source.get_member_explanation(
                language,
                **kwargs
            )

        return explanation




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from threading import local
from contextlib import contextmanager
from warnings import warn
from cocktail.modeling import DictWrapper, ListWrapper
from cocktail.pkgutils import get_full_name

_thread_data = local()


class UndefinedTranslation(object):

    def __nonzero__(self):
        return False

    def __str__(self):
        return "Undefined translation"

undefined = UndefinedTranslation()


@contextmanager
def language_context(language):

    prev_language = get_language()

    try:
        set_language(language)
        yield prev_language
    finally:
        set_language(prev_language)

def get_language():
    return getattr(_thread_data, "language", None)

def set_language(language):
    setattr(_thread_data, "language", language)


class TranslationsRepository(DictWrapper):

    def __init__(self):
        self.__translations = {}
        DictWrapper.__init__(self, self.__translations)

    def __setitem__(self, language, translation):
        self.__translations[language] = translation
        translation.language = language

    def define(self, obj, **strings):

        for language, string in strings.iteritems():
            translation = self.__translations.get(language)

            if translation is None:
                translation = Translation()
                self.__translations[language] = translation
            
            translation[obj] = string

    def __call__(self, obj,
        language = None,
        default = "",
        chain = None,
        **kwargs):
        
        value = ""

        if language is None:
            language = get_language()

        # Translation method
        translation_method = getattr(obj, "__translate__", None)

        if translation_method:
            value = translation_method(language, **kwargs)
        
        # Translation key
        if not value:
            translation = self.__translations.get(language, None)
            if translation is not None:
                value = translation(obj, **kwargs)

        # Per-type translation
        if not value \
        and not isinstance(obj, basestring) \
        and hasattr(obj.__class__, "mro"):

            for cls in obj.__class__.mro():
                try:
                    type_key = get_full_name(cls) + "-instance"
                except:
                    type_key = cls.__name__ + "-instance"
                        
                value = self(type_key, language, instance = obj, **kwargs)
                
                if value:
                    break
        
        # Custom translation chain
        if not value and chain is not None:
            value = self(chain, language, default, **kwargs)

        # Object specific translation chain
        if not value:
            object_chain = getattr(obj, "translation_chain", None)
            if object_chain is not None:
                value = self(object_chain, language, default, **kwargs)

        # Explicit default
        if not value and default != "":
            value = unicode(default)

        return value

translations = TranslationsRepository()

def translate(*args, **kwargs):
    warn(
        "translate() is deprecated, use translations() instead",
        DeprecationWarning,
        stacklevel = 2,
    )
    return translations(*args, **kwargs)


class Translation(DictWrapper):

    language = None

    def __init__(self):
        self.__strings = {}
        DictWrapper.__init__(self, self.__strings)

    def __setitem__(self, obj, string):
        self.__strings[obj] = string

    def __call__(self, obj, **kwargs):
        
        try:
            value = self.__strings.get(obj, "")
        except TypeError:
            return ""
    
        if value:

            # Custom python expression
            if callable(value):
                value = value(**kwargs)

            # String formatting
            elif kwargs:
                value = value % kwargs

        return value


CA_APOSTROPHE_LETTERS = u"haàeèéiíoòóuú"

def ca_apostrophe(word):
    return word and word[0].lower() in CA_APOSTROPHE_LETTERS

def ca_possessive(text):
    if ca_apostrophe(text):
        return u"d'" + text
    else:
        return u"de " + text

def create_join_function(language, sep1, sep2):

    def join(sequence):
        if not isinstance(sequence, (list, ListWrapper)):
            sequence = list(sequence)

        if len(sequence) > 1:
            return sep1.join(sequence[:-1]) + sep2 + sequence[-1]
        else:
            return sequence[0]

    join.func_name = language + "_join"
    return join

ca_join = create_join_function("ca", u", ", u" i ")
es_join = create_join_function("es", u", ", u" y ")
en_join = create_join_function("en", u", ", u" and ")
de_join = create_join_function("en", u", ", u" und ")

def plural2(count, singular, plural):
    if count == 1:
        return singular
    elif count > 1:
        return plural




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from cocktail.translations import translations
from cocktail.html import Element

NOT_ACCESSIBLE = 0
ACCESSIBLE = 1
ACCESSIBLE_DESCENDANTS = 2


class TreeView(Element):

    tag = "ul"
    root = None
    root_visible = True
    create_empty_containers = False
    filter_item = None
    __item_access = None

    def _is_accessible(self, item):
        
        if self.__item_access is None:
            self.__item_access = {}
        
        accessibility = self.__item_access.get(item)

        if accessibility is None:

            if self.filter_item(item):
                accessibility = ACCESSIBLE
            elif any(
                self._is_accessible(child)
                for child in self.get_child_items(item)
            ):
                accessibility = ACCESSIBLE_DESCENDANTS
            else:
                accessibility = NOT_ACCESSIBLE

            self.__item_access[item] = accessibility

        return accessibility

    def _ready(self):
        Element._ready(self)
        
        if self.root is not None:
            if self.root_visible:
                if not self.filter_item or self._is_accessible(self.root):
                    self.root_entry = self.create_entry(self.root)
                    self.append(self.root_entry)
            else:
                self._fill_children_container(
                    self,
                    self.root,
                    self.get_child_items(self.root)
                )

        self.__item_access = None

    def create_entry(self, item):
        
        entry = Element("li")
    
        entry.label = self.create_label(item)
        entry.append(entry.label)

        children = self.get_child_items(item)

        if self.create_empty_containers or children:
            entry.container = self.create_children_container(item, children)
            entry.append(entry.container)

        return entry

    def create_label(self, item):
        label = Element("div")
        label.add_class("entry_label")
        label.append(self.get_item_label(item))

        if self.filter_item \
        and self._is_accessible(item) != ACCESSIBLE:
            label.add_class("filtered")
        else:
            url = self.get_item_url(item)
            if url is not None:
                label.tag = "a"
                label["href"] = url

        return label

    def get_item_label(self, item):
        return translations(item)

    def create_children_container(self, item, children):        
        container = Element("ul")
        container.collapsible = True
        self._fill_children_container(container, item, children)
        return container

    def _fill_children_container(self, container, item, children):
        if children:
            for child in children:
                if not self.filter_item or self._is_accessible(child):
                    container.append(self.create_entry(child))

    def get_child_items(self, parent):
        return parent.children
        
    def get_item_url(self, content_type):
        return None




#-*- coding: utf-8 -*-
u"""
Profiler tool for CherryPy request handlers.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""
from cProfile import runctx
from pstats import Stats
from threading import Lock
from pickle import dumps
from os.path import join
import cherrypy

_lock = Lock()
_request_id = 0

def handler_profiler(stats_path = None):

    handler = cherrypy.request.handler

    def profiled_handler(*args, **kwargs):
        
        global _request_id

        # Acquire a unique identifier for the request
        if stats_path:
            _lock.acquire()
            try:
                _request_id += 1
                id = _request_id
            finally:
                _lock.release()

        # Create the execution context for the profiler
        local_context = {
            "handler": handler,
            "args": args,
            "kwargs": kwargs,
            "rvalue": None
        }

        # Run the handler for the current request through the profiler.
        # Profile data is either shown on standard output or stored on the
        # indicated file.
        try:
            runctx(
                "rvalue = handler(*args, **kwargs)",
                globals(),
                local_context,
                join(stats_path, "%s.stats" % id) if stats_path else None
            )

        # Store request data, to match profiler stats against their context
        finally:
            if stats_path:
                f = open(join(stats_path, "%s.context" % id), "w")
                try:
                    request_context = {
                        "path_info": cherrypy.request.path_info,
                        "query_string": cherrypy.request.query_string,
                        "headers": cherrypy.request.headers,
                        "args": args,
                        "kwargs": kwargs
                    }
                    f.write(dumps(request_context))
                finally:
                    f.close()

        return local_context["rvalue"]

    cherrypy.request.handler = profiled_handler

cherrypy.tools.handler_profiler = cherrypy.Tool(
    'before_handler',
    handler_profiler
)

if __name__ == "__main__":
    
    import sys
    from optparse import OptionParser
    from pprint import pprint
    from pickle import loads

    parser = OptionParser()
    parser.add_option("-s", "--sort",
        help = "The sorting order for the profile data. "
        "Accepts the same values as the pstats.Stats.sort_stats() "
        "method."
    )
    parser.add_option("-t", "--top",
        type = "int",
        help = "Limits data to the first N rows."
    )
    parser.add_option("-p", "--path",
        default = ".",
        help = "The folder where the profiling information resides."
    )

    options, args = parser.parse_args()
    
    if not args:
        sys.stderr.write("Need one or more profile identifiers\n")
        sys.exit(1)

    for arg in args:
        
        # Load context
        context_path = join(options.path, "%s.context" % arg)
        context_file = open(context_path, "r")
        try:
            context = loads(context_file.read())
        finally:
            context_file.close()

        # Load stats
        stats_path = join(options.path, "%s.stats" % arg)
        stats = Stats(stats_path)

        if options.sort:
            stats.sort_stats(options.sort)
        
        print "Context"
        print "-" * 80
        print pprint(context)
        print
        print "Profile"
        print "-" * 80
        stats.print_stats(options.top or None)




#-*- coding: utf-8 -*-
u"""
Declarative definition of data models and validation rules.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from cocktail.schema.member import (
    Member,
    DynamicDefault
)
from cocktail.schema.schema import Schema
from cocktail.schema.rangedmember import RangedMember
from cocktail.schema.schemacollections import (
    Collection, add, remove,
    RelationCollection,
    RelationList,
    RelationSet,
    RelationOrderedSet
)
from cocktail.schema.schemamappings import (
    Mapping,
    RelationMapping
)
from cocktail.schema.schemarelations import RelationMember
from cocktail.schema.schemareference import Reference
from cocktail.schema.schemastrings import String
from cocktail.schema.schemanumbers import (
    Number,
    Integer,
    Decimal,
    Float
)
from cocktail.schema.schemabooleans import Boolean
from cocktail.schema.schemadates import (
    BaseDateTime,
    DateTime,
    Date,
    Time
)
from cocktail.schema.errorlist import ErrorList
from cocktail.schema.accessors import (
    get_accessor,
    get,
    set,
    MemberAccessor,
    AttributeAccessor,
    DictAccessor
)
from cocktail.schema.schemaobject import (
    SchemaObject,
    SchemaClass,
    SchemaObjectAccessor
)
from cocktail.schema.adapter import (
    reference, shallow, deep,
    Adapter,
    RuleSet,
    Rule,
    Copy,
    Exclusion,
    Split,
    Join
)
from cocktail.schema.validationcontext import ValidationContext
from cocktail.schema.differences import diff




#-*- coding: utf-8 -*-
u"""
Provides classes to describe members that take dates and times as values.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			April 2008
"""
import datetime
from calendar import monthrange
from cocktail.schema.schema import Schema
from cocktail.schema.rangedmember import RangedMember
from cocktail.schema.schemanumbers import Integer
from cocktail.translations import translations

def get_max_day(context):

    date = context.validable

    if date.year is not None and (0 < date.month <= 12):
        return monthrange(date.year, date.month)[1]

    return None

class BaseDateTime(Schema, RangedMember):
    """Base class for all members that handle date and/or time values."""
    _is_date = False
    _is_time = False
   
    def __init__(self, *args, **kwargs):
        
        Schema.__init__(self, *args, **kwargs)
        RangedMember.__init__(self)

        day_kw = {"name": "day", "min": 1, "max": get_max_day}
        month_kw = {"name": "month", "min": 1, "max": 12}
        year_kw = {"name": "year"}
        hour_kw = {"name": "hour", "min": 0, "max": 23}
        minute_kw = {"name": "minute", "min": 0, "max": 59}
        second_kw = {"name": "second", "min": 0, "max": 59}

        day_properties = kwargs.pop("day_properties", None)
        month_properties = kwargs.pop("month_properties", None)
        year_properties = kwargs.pop("year_properties", None)
        hour_properties = kwargs.pop("hour_properties", None)
        minute_properties = kwargs.pop("minute_properties", None)
        second_properties = kwargs.pop("second_properties", None)

        if day_properties:
            day_kw.update(day_properties)

        if month_properties:
            month_kw.update(month_properties)

        if year_properties:
            year_kw.update(year_properties)

        if hour_properties:
            hour_kw.update(hour_properties)

        if minute_properties:
            minute_kw.update(minute_properties)

        if second_properties:
            second_kw.update(second_properties)

        if self._is_date:
            self.add_member(Integer(**day_kw))
            self.add_member(Integer(**month_kw))
            self.add_member(Integer(**year_kw))
        
        if self._is_time:
            self.add_member(Integer(**hour_kw))
            self.add_member(Integer(**minute_kw))
            self.add_member(Integer(**second_kw))


class DateTime(BaseDateTime):
    type = datetime.datetime
    _is_date = True
    _is_time = True
    translate_value = translations


class Date(BaseDateTime):
    type = datetime.date
    _is_date = True
    translate_value = translations


class Time(BaseDateTime):
    type = datetime.time
    _is_time = True




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from simplejson import dumps
from cocktail.translations import get_language
from cocktail.html import Element, templates
from cocktail.html.databoundcontrol import DataBoundControl


class TinyMCE(Element, DataBoundControl):
    
    tag = None

    def __init__(self, *args, **kwargs):
        Element.__init__(self, *args, **kwargs)
        DataBoundControl.__init__(self)
        self.tinymce_params = {}
        self.add_resource(
            "/resources/scripts/tinymce/jscripts/tiny_mce/tiny_mce.js")

    def _build(self):
        self.textarea = templates.new("cocktail.html.TextArea")
        self.append(self.textarea)
        self.binding_delegate = self.textarea
        
    def _ready(self):

        Element._ready(self)

        params = self.tinymce_params.copy()
        params.setdefault("language", get_language())
        params["mode"] = "exact"
        params["elements"] = self.textarea.require_id()

        init_script = Element("script")
        init_script["type"] = "text/javascript"
        init_script.append("tinyMCE.init(%s)" % dumps(params))
        self.append(init_script)
    
    def _get_value(self):
        return self.textarea.value

    def _set_value(self, value):
        self.textarea.value = value

    value = property(_get_value, _set_value, doc = """
        Gets or sets the content of the rich text editor.
        @type: str
        """)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
from cocktail.modeling import ListWrapper, SetWrapper
from cocktail.translations import translations
from cocktail import schema
from cocktail.html import Element
from cocktail.html.databoundcontrol import DataBoundControl


class Selector(Element, DataBoundControl):

    name = None
    items = None
    __value = None
    
    empty_option_displayed = True
    empty_value = ""
    empty_label = "---"
   
    def __init__(self, *args, **kwargs):
        Element.__init__(self, *args, **kwargs)
        DataBoundControl.__init__(self)
        self._is_selected = lambda item: False

    def _ready(self):

        Element._ready(self)
        item_translator = None

        if self.member:
        
            if self.items is None:
                self.items = self._get_items_from_member(self.member)

            if isinstance(self.member, schema.Collection):
                if self.member.items:
                    item_translator = self.member.items.translate_value
            else:
                item_translator = self.member.translate_value
        
        self._item_translator = (
            item_translator
            or (lambda item, **kw: translations(item, **kw))
        )
        
        self._fill_entries()

    def _get_items_from_member(self, member):
        
        if isinstance(member, schema.Collection):
            enumeration = member.items.enumeration if member.items else None
        else:
            enumeration = member.enumeration
        
        if enumeration is not None:
            enumeration = member.resolve_constraint(enumeration, None)
            if enumeration is not None:
                return enumeration

        if isinstance(member, schema.Boolean):
            return (True, False)

        elif isinstance(member, schema.Number):
            if member.min is not None and member.max is not None:
                return range(member.min, member.max + 1)

        elif isinstance(member, schema.RelationMember):
            if getattr(member, "is_persistent_relation", False):
                items = member.select_constraint_instances(
                    parent = self.persistent_object or self.data
                )
                
                order = None

                if isinstance(member, schema.Reference):
                    order = member.default_order
                elif isinstance(member, schema.Collection):
                    order = member.items.default_order

                if order:
                    items.add_order(order)

                return items
                
            elif isinstance(member, schema.Collection):
                if member.items:
                    return self._get_items_from_member(member.items)
        
        return ()

    def _fill_entries(self):
        
        if self.empty_option_displayed:
            entry = self.create_entry(
                self.empty_value,
                self.empty_label,
                self.value is None
            )
            self.append(entry)

        if hasattr(self.items, "iteritems"):
            for value, label in self.items.iteritems():
                value = self.get_item_value(value)
                entry = self.create_entry(
                    value,
                    label,
                    self._is_selected(value)
                )
                self.append(entry)
        else:
            for item in self.items:
                value = self.get_item_value(item)
                label = self.get_item_label(item)
                entry = self.create_entry(
                    value,
                    label,
                    self._is_selected(value)
                )
                self.append(entry)
    
    def get_item_value(self, item):
       
        member = (
            self.member
            and (
                isinstance(self.member, schema.Collection)
                and self.member.items
            )
            or self.member
        )

        if member:            
            try:
                return member.serialize_request_value(item)
            except:
                pass
        
        return getattr(item, "id", None) or str(item)

    def get_item_label(self, item):
        return self._item_translator(item)
    
    def create_entry(self, value, label, selected):
        pass

    def _get_value(self):
        return self.__value

    def _set_value(self, value):
        self.__value = value

        if value is None:
            self._is_selected = lambda item: False
        elif isinstance(value, (list, tuple, set, ListWrapper, SetWrapper)):
            selection = set(self.get_item_value(item) for item in value)
            self._is_selected = lambda item: item in selection
        else:
            selection = self.get_item_value(value)
            self._is_selected = lambda item: item == selection

    value = property(_get_value, _set_value, doc = """
        Gets or sets the active selection for the selector.
        """)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from time import time
from itertools import chain
from cocktail.styled import styled
from cocktail.modeling import getter, ListWrapper
from cocktail.language import get_content_language
from cocktail.schema import Member, expressions, SchemaObjectAccessor
from cocktail.schema.io import export_file
from cocktail.schema.expressions import TranslationExpression
from cocktail.persistence.index import Index

query_style = lambda t: styled(t.ljust(80), "white", "blue")
query_attrib_style = lambda k, v: (
    " " * 4
    + styled((k + ":").ljust(15), style = "bold")
    + str(v)
)
query_cached_style = lambda t: styled(" " * 4 + t, "white", "green")
query_phase_style = lambda t: " " * 4 + styled(t, style = "underline")
query_eval_style = (
    lambda expr: " " * 4 + styled("Eval:", "magenta") + str(expr)
)
query_initial_dataset_style = (
    lambda dataset:
        " " * 4 + "Initial dataset: "
        + styled(len(dataset), style = "bold")
)
query_resolve_style = (
    lambda expr: " " * 4 + styled("Resolve:", "bright_green") + str(expr)
)
query_results_style = (
    lambda n: " " * 4 + styled(n, style = "bold")
)
query_timing_style = lambda s: " " * 4 + ("%.4fs" % s)

inherit = object()

class Query(object):
    """A query over a set of persistent objects."""

    verbose = False

    def __init__(self,
        type,
        filters = None,
        order = None,
        range = None,
        base_collection = None,
        cached = True):

        self.__type = type
        self.__filters = None
        self.__order = None
        self.__base_collection = None

        self.__cached_results = None
        self.__cached_results_sorted = False

        self.filters = filters or []
        self.order = order or []
        self.range = range
        self.base_collection = base_collection
        self.cached = cached

    def __repr__(self):
        return (
            "Query("
            "type = %s, "
            "filters = %r, "
            "order = %r, "
            "range = %r, "
            "cached = %r)" % (
                self.__type.full_name,
                self.__filters,
                self.__order,
                self.__range,
                self.cached
            )
        )

    def discard_results(self):
        self.__cached_results = None
        self.__cached_results_sorted = False

    @getter
    def type(self):
        """The base type of the instances returned by the query.
        @type: L{PersistentObject<cocktail.persistence.persistentobject.PersistentObject>}
            subclass
        """
        return self.__type

    def _get_base_collection(self):
        if self.__base_collection is None:
            return self.type.index.itervalues()
        else:
            return self.__base_collection
    
    def _set_base_collection(self, value):

        if value is None and not self.type.indexed:
            raise TypeError("An indexed persistent class is required")

        self.discard_results()
        self.__base_collection = value

    base_collection = property(_get_base_collection, _set_base_collection,
        doc = """The base set of persistent objects that the query operates on.
        When set to None, the query works with the full set of instances for
        the selected type.
        @type: L{PersistentObject<cocktail.persistence.PersistentObject>}
        """)

    def _get_filters(self):
        return self.__filters
    
    def _set_filters(self, value):
        
        self.discard_results()

        if value is None:
            self.__filters = []
        else:
            if isinstance(value, expressions.Expression):
                self.__filters = [value]
            elif isinstance(value, dict):
                self.__filters = [
                    self.type[k].equal(v) for k, v in value.iteritems()
                ]
            else:
                self.__filters = list(value)

    filters = property(_get_filters, _set_filters, doc = """
        An optional set of constraints that all instances produced by the query
        must fulfill. All expressions are joined using a logical 'and'
        expression.
        @type filters: L{Expression<cocktail.schema.expressions.Expression>}
            list
        """)
    
    def add_filter(self, filter):
        
        self.discard_results()

        if self.filters is None:
            self.filters = [filter]
        else:
            self.filters.append(filter)

    def _get_order(self):
        return self.__order or []

    def _set_order(self, value):

        self.discard_results()

        if value is None:
            self.__order = []
        else:
            if isinstance(value, (basestring, expressions.Expression)):
                value = value,

            order = []

            for criteria in value:
                order.append(self._normalize_order_criteria(criteria))

            self.__order = order

    order = property(_get_order, _set_order, doc = """            
        Specifies the order in which instances are returned. Can take both
        single and multiple sorting criteria. Criteria can be specified using
        member names or references to L{Member<cocktail.schema.member.Member>}
        objects.
        
        When specified using a string, an optional '+' or '-' prefix can be
        used to choose between ascending (default) and descending order,
        respectively. When using member references, the same can be
        accomplished by wrapping the reference inside a
        L{PositiveExpression<cocktail.schema.expressions.PositiveExpression>}
        or a L{NegativeExpression<cocktail.schema.expressions.NegativeExpression>}.

        @type: str,
            str collection,
            L{Expression<cocktail.schema.expressions.Expression>},
            L{Expression<cocktail.schema.expressions.Expression>} collection
        """)

    def add_order(self, criteria):
        self.discard_results()
        self.__order.append(self._normalize_order_criteria(criteria))

    def _normalize_order_criteria(self, criteria):
        
        if isinstance(criteria, basestring):
            if not criteria:
                raise ValueError(
                    "An empty string is not a valid query filter"
                )
            
            wrapper = None

            if criteria[0] == "+":
                criteria = criteria[1:]
                wrapper = expressions.PositiveExpression
            elif criteria[0] == "-":
                criteria = criteria[1:]
                wrapper = expressions.NegativeExpression

            criteria = self.type[criteria]

            if wrapper:
                criteria = wrapper(criteria)
        
        elif not isinstance(criteria, expressions.Expression):
            raise TypeError(
                "Query.order expected a string or Expression, got "
                "%s instead" % criteria
            )

        if not isinstance(criteria,
            (expressions.PositiveExpression,
            expressions.NegativeExpression)
        ):
            criteria = expressions.PositiveExpression(criteria)

        return criteria

    def _get_range(self):
        return self.__range

    def _set_range(self, value):

        if value is None:
            self.__range = value
        else:
            if not isinstance(value, tuple) \
            or len(value) != 2 \
            or not isinstance(value[0], int) \
            or not isinstance(value[1], int):
                raise TypeError("Invalid query range: %s" % value)
            
            if value[0] < 0 or value[1] < 0:
                raise ValueError(
                    "Negative indexes not supported on query ranges: %d, %d"
                    % value
                )
            
            self.__range = value

    range = property(_get_range, _set_range, doc = """        
        Limits the set of matching instances to the given integer range. Ranges
        start counting from zero. Negative indexes or None values are not
        allowed.
        @type: (int, int) tuple
        """)

    # Execution
    #--------------------------------------------------------------------------    
    def execute(self, _sorted = True):

        if self.verbose:
            print query_style("Query")
            print query_attrib_style("type", self.type.full_name)

            if self.filters:
                print query_attrib_style("filters", self.filters)

            if self.order:
                print query_attrib_style("order", self.order)

            if self.range:
                print query_attrib_style("range", self.range)
            
        # Try to make use of cached results
        if self.cached and self.__cached_results is not None:
            dataset = self.__cached_results
            
            if self.verbose:
                print query_cached_style("cached")

        # New data set
        else:
            # Object universe
            if self.__base_collection is None:
                dataset = self.type.keys
            else:
                dataset = (obj.id for obj in self.__base_collection)

            # Apply filters
            if self.verbose:
                start = time()
                print query_phase_style("Applying filters")

            if self.__filters:
                dataset = self._apply_filters(dataset)

            if self.verbose:
                print query_timing_style(time() - start)

        # Apply order
        if _sorted and not (self.cached and self.__cached_results_sorted):

            if self.verbose:
                start = time()
                print query_phase_style("Applying order")
            
            # Preserve ordering when selecting items from a custom ordered
            # collection
            if not self.__order and isinstance(
                self.__base_collection,
                (list, tuple, ListWrapper)
            ):
                dataset = [obj.id
                           for obj in self.__base_collection
                           if obj.id in dataset]
            else:
                dataset = self._apply_order(dataset)

            if self.verbose:
                print query_timing_style(time() - start)

        # Store results for further requests
        if self.cached:
            self.__cached_results = dataset
            self.__cached_results_sorted = _sorted
        
        # Apply ranges
        if _sorted:
            if self.verbose:
                start = time()
                print query_phase_style("Applying range")
            
            dataset = self._apply_range(dataset)

            if self.verbose:
                print query_timing_style(time() - start)

        if self.verbose:
            print

        return dataset

    def _apply_filters(self, dataset):
 
        type_index = self.type.index
        
        if self.verbose:
            print query_initial_dataset_style(dataset)

        for expr, custom_impl in self._get_execution_plan(self.__filters):

            if not isinstance(dataset, set):
                dataset = set(dataset)

            # As soon as the matching set is reduced to an empty set
            # there's no point in applying any further filter
            if not dataset:
                break
 
            # Default filter implementation. Used when no custom transformation
            # function is provided, or if the dataset has been reduced to a
            # single item (to spare the intersection between the results for
            # all remaining filters)
            if custom_impl is None or len(dataset) == 1:

                if self.verbose:
                    print query_eval_style(expr)

                dataset.intersection_update(
                    id
                    for id in dataset
                    if expr.eval(type_index[id], SchemaObjectAccessor)
                )
                
            # Custom filter implementation
            else:
                
                if self.verbose:
                    print query_resolve_style(expr)

                dataset = custom_impl(dataset)

            if self.verbose:
                print query_results_style(len(dataset))

        return dataset

    def _get_execution_plan(self, filters):
        """Create an optimized execution plan for the given set of filters."""

        plan = []
        And = expressions.AndExpression

        for filter in filters:
            # Flatten regular AND expressions
            if isinstance(filter, And):
                for operand in filter.operands:
                    plan.append((operand.resolve_filter(self), operand))
            else:
                plan.append((filter.resolve_filter(self), filter))

        plan.sort()

        for resolution, filter in plan:
            yield (filter, resolution[1])
    
    def _get_expression_index(self, expr, member = None):

        # Expressions can override normal indexes and supply their own
        if isinstance(expr, Member):
            index = self._get_member_index(expr)
        else:
            index = expr.index
                
        # Otherwise, just use the one provided by the evaluated member
        if index is None and member is not None:
            index = self._get_member_index(member)

        return index

    def _get_member_index(self, member):

        if member.indexed:
            if member.primary:
                return self.__type.index
            else:
                return member.index

        return None
    
    def _apply_order(self, dataset):
 
        order = []

        # Wrap the translated members into a TranslationExpression
        for criteria in self.__order:
            member = criteria.operands[0]
            if isinstance(member, Member) and member.translated:
                order.append(criteria.__class__(
                    member.translated_into(get_content_language())
                ))
            else:
                order.append(criteria)

        # Force a default order on queries that don't specify one, but request
        # a dataset range
        if not order:
            if self.range:
                order = [self.__type.primary_member.positive()]
            else:
                return dataset 
        
        # Optimized case: single indexed member, or a member that is both
        # required and unique followed by other members
        expr = order[0].operands[0]

        if isinstance(expr, TranslationExpression):
            language = expr.operands[1].eval(None)
            expr = expr.operands[0]

        index = self._get_expression_index(expr)

        if index is not None and (
            len(order) == 1
            or (isinstance(expr, Member) and expr.unique and expr.required)
        ):            
            if not isinstance(dataset, set):
                dataset = set(dataset)

            if isinstance(expr, Member) and expr.primary:
                sequence = index.iterkeys()
            else:
                if expr.translated:
                    sequence = (id
                                for key, id in index.iteritems()
                                if key[0] == language)
                else:
                    sequence = index.itervalues()

            dataset = [id
                       for id in sequence
                       if id in dataset]

            if isinstance(order[0], expressions.NegativeExpression):
                dataset.reverse()

            return dataset

        # Optimized case: indexes available for all involved criteria
        indexes = []
        
        for criteria in order:            
            expr = criteria.operands[0]

            if isinstance(expr, TranslationExpression):
                language = expr.operands[1].eval(None)
                expr = expr.operands[0]
            else:
                language = None

            index = self._get_expression_index(expr)
            if index is None:
                break            
            reversed = isinstance(criteria, expressions.NegativeExpression)
            indexes.append((expr, index, reversed, language))
        else:
            ranks = {}
            
            def add_rank(id, rank):
                item_ranks = ranks.get(id)
                if item_ranks is None:
                    item_ranks = []
                    ranks[id] = item_ranks
                item_ranks.append(rank)

            if not isinstance(dataset, set):
                dataset = set(dataset)

            for expr, index, reversed, language in indexes:
             
                if isinstance(index, Index):
                    for i, key in enumerate(index):
                        if reversed:
                            i = -i
                        if expr.translated and key[0] != language:
                            continue
                        for id in index[key]:
                            if id in dataset:
                                add_rank(id, i)
                else:                    
                    if isinstance(expr, Member) and expr.primary:
                        sequence = index.iterkeys()
                    else:
                        if expr.translated:
                            sequence = (id
                                for key, id in index.iteritems()
                                if key[0] == language)
                        else:
                            sequence = index.itervalues()

                    for i, id in enumerate(sequence):
                        if id in dataset:
                            if reversed:
                                i = -i                        
                            add_rank(id, i)

            return sorted(dataset, key = ranks.__getitem__)

        # Proceed with the brute force approach
        if not isinstance(dataset, list):
            dataset = list(dataset)

        order_expressions = []
        descending = []
        
        for expr in order:
            order_expressions.append(expr.operands[0])
            descending.append(isinstance(expr, expressions.NegativeExpression))

        type_index = self.type.index

        entries = []

        for id in dataset:
            sorting_values = []
            for expr in order_expressions:
                value = expr.eval(type_index[id], SchemaObjectAccessor)
                if hasattr(value, "get_ordering_key"):
                    value = value.get_ordering_key()
                sorting_values.append(value)
            entries.append([id, sorting_values])

        def compare(entry_a, entry_b):
            
            values_a = entry_a[1]
            values_b = entry_b[1]

            for expr, desc, value_a, value_b in zip(
                order_expressions,
                descending,
                values_a,
                values_b
            ):            
                if desc:
                    value_a, value_b = value_b, value_a

                if (value_a is None and value_b is None) \
                or value_a == value_b:
                    pass
                elif value_a is None:
                    return -1
                elif value_b is None:
                    return 1
                elif value_a < value_b:
                    return -1
                elif value_a > value_b:
                    return 1

            return 0

        entries.sort(cmp = compare)
        return [entry[0] for entry in entries]

    def _apply_range(self, dataset):

        r = self.range
        
        if dataset and r:
            dataset = dataset[r[0]:r[1]]

        return dataset

    def __iter__(self):
        type_index = self.type.index
        for id in self.execute():
            yield type_index[id]
        
    def __len__(self):
        
        if self.filters:
            return len(self.execute(_sorted = False))
        else:
            if self.__base_collection is None:
                return len(self.type.keys)
            else:
                return len(self.__base_collection)
    
    def __notzero__(self):
        # TODO: Could be optimized to look for just one match on each filter
        return bool(self.execute(_sorted = False))

    def __contains__(self, item):
        # TODO: This too could be optimized
        return item.id in self.execute()

    def __getitem__(self, index):
        
        # Retrieve a slice
        if isinstance(index, slice):
            if index.step is not None and index.step != 1:
                raise ValueError("Can't retrieve a query slice using a step "
                        "different than 1")
            
            return self.select(range = (index.start, index.stop))

        # Retrieve a single item
        else:
            results = self.execute()
            return self.type.index[results[index]]

    def select(self,
        filters = inherit,
        order = inherit,
        range = inherit):
        
        child_query = self.__class__(
            self.__type,
            self.filters if filters is inherit else filters + self.filters,
            self.order if order is inherit else order,
            self.range if range is inherit else range,
            self.__base_collection)

        if filters is None:
            child_query.__cached_results = self.__cached_results
            child_query.__cached_results_sorted = order is None \
                and self.__cached_results_sorted

        return child_query

    def delete_items(self):
        """Delete all items matched by the query."""
        type_index = self.type.index
        for id in list(self.execute(_sorted = False)):
            item = type_index.get(id)
            if item is not None:
                item.delete()

    # Exportation
    #------------------------------------------------------------------------------
    def export_file(self, dest, mime_type = None, members = None, **kwargs):
        """Exports the query to a file"""
        export_file(self, dest, self.type, mime_type, members, **kwargs)
        


# Custom expression resolution
#------------------------------------------------------------------------------
def _get_filter_info(filter):

    if isinstance(filter, Member):
        member = filter
        index = member.index
    else:
        member = filter.operands[0] \
            if filter.operands and isinstance(filter.operands[0], Member) \
            else None
    
        index = member.index if member is not None else None

        if index is None:
            index = filter.index

    unique = not isinstance(index, Index)
    return member, index, unique

def _get_index_value(member, value):
    value = member.get_index_value(value)
    if member.translated:
        value = (get_content_language(), value)
    return value

def _expression_resolution(self, query):
    return ((0, 0), None)
    
expressions.Expression.resolve_filter = _expression_resolution

def _equal_resolution(self, query):

    member, index, unique = _get_filter_info(self)     

    if index is None:
        return ((0, -1), None)

    elif unique:
        order = (-2, -1)

        if member and member.primary:
            def impl(dataset):
                id = _get_index_value(member, self.operands[1].value)
                return set([id]) if id in dataset else set()
        else:
            def impl(dataset):
                value = _get_index_value(member, self.operands[1].value)
                match = index.get(value)
                            
                if match is None:
                    return set()
                else:
                    return set([match])
    else:
        order = (-1, -1)
        def impl(dataset):
            value = _get_index_value(member, self.operands[1].value)
            subset = index[value]
            dataset.intersection_update(subset)
            return dataset
    
    return (order, impl)

expressions.EqualExpression.resolve_filter = _equal_resolution

def _not_equal_resolution(self, query):

    member, index, unique = _get_filter_info(self)     

    if index is None:
        return ((0, 0), None)
    elif unique:
        order = (-2, 0)
        
        if member and member.primary:
            def impl(dataset):
                id = _get_index_value(member, self.operands[1].value)
                dataset.discard(id)
                return dataset
        else:
            def impl(dataset):
                value = _get_index_value(member, self.operands[1].value)
                index_value = index.get(value)
                if index_value is not None:
                    dataset.discard(index_value)
                return dataset
    else:
        order = (-1, -1)
        def impl(dataset):
            value = _get_index_value(member, self.operands[1].value)
            subset = index[value]
            dataset.difference_update(subset)
            return dataset
    
    return (order, impl)

expressions.NotEqualExpression.resolve_filter = _not_equal_resolution

def _greater_resolution(self, query):

    member, index, unique = _get_filter_info(self)

    if index is None:
        return ((0, 0), None)
    else:
        exclude_end = isinstance(self, expressions.GreaterExpression)
        
        def impl(dataset):
            value = _get_index_value(member, self.operands[1].value)
            method = index.keys if member and member.primary else index.values
            subset = method(min = value, excludemin = exclude_end)
            dataset.intersection_update(subset)
            return dataset

        return ((-2 if unique else -1, 0), impl)

expressions.GreaterExpression.resolve_filter = _greater_resolution
expressions.GreaterEqualExpression.resolve_filter = _greater_resolution

def _lower_resolution(self, query):

    member, index, unique = _get_filter_info(self)

    if index is None:
        return ((0, 0), None)
    else:
        exclude_end = isinstance(self, expressions.LowerExpression)
        def impl(dataset):
            value = _get_index_value(member, self.operands[1].value)
            method = index.keys if member and member.primary else index.values
            subset = method(max = value, excludemax = exclude_end)
            dataset.intersection_update(subset)
            return dataset

        return ((-2 if unique else -1, 0), impl)

expressions.LowerExpression.resolve_filter = _lower_resolution
expressions.LowerEqualExpression.resolve_filter = _lower_resolution

def _inclusion_resolution(self, query):

    if self.operands and self.operands[0] is expressions.Self:
        
        subset = self.operands[1].eval(None)

        if not self.by_key:
            subset = set(item.id for item in subset)

        def impl(dataset):
            dataset.intersection_update(subset)
            return dataset
        
        return ((-3, 0), impl)
    else:
        return ((0, 0), None)

expressions.InclusionExpression.resolve_filter = _inclusion_resolution

def _exclusion_resolution(self, query):

    if self.operands and self.operands[0] is expressions.Self:
        
        subset = self.operands[1].eval(None)

        if not self.by_key:
            subset = set(item.id for item in subset)

        def impl(dataset):
            dataset.difference_update(subset)
            return dataset
        
        return ((-3, 0), impl)
    else:
        return ((0, 0), None)

expressions.ExclusionExpression.resolve_filter = _exclusion_resolution

# TODO: Provide an index aware custom implementation for StartsWithExpression
expressions.StartsWithExpression.resolve_filter = \
    lambda self, query: ((0, -1), None)

expressions.EndsWithExpression.resolve_filter = \
    lambda self, query: ((0, -1), None)

expressions.ContainsExpression.resolve_filter = \
    lambda self, query: ((0, -2), None)

expressions.MatchExpression.resolve_filter = \
    lambda self, query: ((0, -3), None)

expressions.SearchExpression.resolve_filter = \
    lambda self, query: ((0, -4), None)

def _has_resolution(self, query):

    index = self.relation.index
    unique = not isinstance(index, Index)

    if index is None:
        return ((1, 0), None)
    else:
        def impl(dataset):

            related_subset = \
                self.relation.related_type.select(self.filters).execute()
            
            if related_subset:
                
                subset = set()

                if unique:
                    for related_id in related_subset:
                        referer_id = index.get(related_id)
                        if referer_id is not None:
                            subset.add(referer_id)
                else:
                    for related_id in related_subset:
                        referers = index[related_id]
                        if referers:
                            subset.update(referers)

                dataset.intersection_update(subset)
                return dataset
            else:
                return set()
                
        return ((0, -1), impl)

expressions.HasExpression.resolve_filter = _has_resolution

def _range_intersection_resolution(self, query):

    min_member = (
        self.operands[0]
        if isinstance(self.operands[0], Member)
        else None
    )

    max_member = (
        self.operands[1]
        if isinstance(self.operands[1], Member)
        else None
    )

    if self.index:
        min_index, max_index = self.index
    else:
        min_index = None if min_member is None else min_member.index
        max_index = None if max_member is None else max_member.index
    
    # One or both fields not indexed
    if min_index is None or max_index is None:
        order = (0, 0)
        impl = None
    else:
        def impl(dataset):
            min_value = self.operands[2].value
            if min_member:
                min_value = _get_index_value(min_member, min_value)

            max_value = self.operands[3].value
            if max_member:
                max_value = _get_index_value(max_member, max_value)          
            
            min_method = (
                min_index.keys
                if min_member and min_member.primary
                else min_index.values
            )

            max_method = (
                max_index.keys
                if max_member and max_member.primary
                else max_index.values
            )

            subset = set(max_method(max = None, excludemax = False))
            subset.update(
                max_method(min = min_value, excludemin = self.excludemin)
            )

            if max_value is not None:
                subset.intersection_update(
                    min_method(max = max_value, excludemax = self.excludemax)
                )
            
            dataset.intersection_update(subset)
            return dataset

        # One or both fields have non-unique indexes
        if isinstance(min_index, Index) or isinstance(max_index, Index):
            order = (-1, 0)
        # Both fields have unique indexes
        else:
            order = (-2, 0)
    
    return (order, impl)


expressions.RangeIntersectionExpression.resolve_filter = \
    _range_intersection_resolution

def _isinstance_subset(expression):

    subset = set()

    if isinstance(expression.operands[1], expressions.Constant):
        operand = expression.operands[1].eval()
    else:
        operand = expression.operands[1]

    if isinstance(operand, (tuple, list)):
        models = operand
    else:
        models = list()
        models.append(operand)

    for cls in models:
        if expression.is_inherited:
            subset.update(cls.keys)
        else:
            children = cls.derived_schemas(recursive = False)
            children_subset = set()
            for child in children:
                children_subset.update(child.keys)
            subset.update(set(cls.keys).difference(children_subset))

    return subset

def _isinstance_resolution(self, query):
    # TODO: Implement the resolution for the queries that its first operand is
    #   a Reference

    if self.operands and self.operands[0] is expressions.Self:

        subset = _isinstance_subset(self)

        def impl(dataset):
            dataset.intersection_update(subset)
            return dataset
        
        return ((-3, 0), impl)
    else:
        return ((0, 0), None)

expressions.IsInstanceExpression.resolve_filter = _isinstance_resolution

def _is_not_instance_resolution(self, query):
    # TODO: Implement the resolution for the queries that its first operand is
    #   a Reference

    if self.operands and self.operands[0] is expressions.Self:
        
        subset = _isinstance_subset(self)

        def impl(dataset):
            dataset.difference_update(subset)
            return dataset
        
        return ((-3, 0), impl)
    else:
        return ((0, 0), None)

expressions.IsNotInstanceExpression.resolve_filter = _is_not_instance_resolution

def _descends_from_resolution(self, query):

    if self.operands[0] is expressions.Self:

        def impl(dataset):
            subset = set()
            root = self.operands[1].eval(None)
            relation = self.relation

            if relation._many:
                def descend(item, include_self):
                    if include_self:
                        subset.add(item.id)
                    children = item.get(relation)
                    if children:                
                        for child in children:
                            descend(child, True)
                descend(root, self.include_self)
            else:
                item = root if self.include_self else root.get(relation)

                while item is not None:
                    subset.add(item.id)
                    item = item.get(relation)
        
            dataset.intersection_update(subset)
            return dataset
        
        return ((-3, 0), impl)
    
    else:
        return ((0, 0), None)

expressions.DescendsFromExpression.resolve_filter = _descends_from_resolution




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2009
"""
import buffet
import cherrypy

_rendering_engines = {}

def get_rendering_engine(engine_name):

    engine = _rendering_engines.get(engine_name)

    if engine is None:
        engine_type = buffet.available_engines[engine_name]
        engine_options = \
            cherrypy.request.config.get("rendering.engine_options")
        engine = engine_type(options = engine_options)
        _rendering_engines[engine_name] = engine

    return engine




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
import sys
from warnings import warn

supported_platforms = ["linux2"]

color_map = {
    ("white", "black", None): 0,
    ("white", "black", "bold"): 1,
    ("white", "black", "underline"): 4,
    ("white", "black", "strike_through"): 9,
    ("black", "white", None): 7,
    ("black", "black", None): 30,
    ("red", "black", None): 31,
    ("green", "black", None): 32,
    ("brown", "black", None): 33,
    ("blue", "black", None): 34,
    ("violet", "black", None): 35,
    ("turquoise", "black", None): 36,
    ("light_gray", "black", None): 37,
    ("white", "red", None): 41,
    ("white", "green", None): 42,
    ("white", "brown", None): 43,
    ("white", "blue", None): 44,
    ("white", "violet", None): 45,
    ("white", "turquoise", None): 46,
    ("white", "light_gray", None): 47,
    ("dark_gray", "black", None): 90,
    ("magenta", "black", None): 91,
    ("bright_green", "black", None): 92,
    ("yellow", "black", None): 93,
    ("slate_blue", "black", None): 94,
    ("pink", "black", None): 95,
    ("cyan", "black", None): 96,
    ("white", "dark_gray", None): 100,
    ("white", "magenta", None): 101,
    ("white", "bright_green", None): 102,
    ("white", "yellow", None): 103,
    ("white", "slate_blue", None): 104,
    ("white", "pink", None): 105,
    ("white", "cyan", None): 106,
    ("white", "white", None): 107
}

if sys.platform in supported_platforms:
    
    def styled(
        string,
        foreground = "white",
        background = "black",
        style = None):
        
        key = (foreground, background, style)
        code = color_map.get(key)

        if code is None:
            warn("Can't print using the requested style: %s" % key)
            return string
        else:
            return "\033[0;%dm%s\033[m" % (code, string)

else:
    def styled(string, foreground = None, background = None, style = None):
        if not isinstance(string, basestring):
            string = str(string)
        return string




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from cocktail.html import Element, templates
from cocktail.html.selector import Selector

CheckBox = templates.get_class("cocktail.html.CheckBox")


class CheckList(Selector):

    empty_option_displayed = False

    def _ready(self):

        if not self.name and self.data_display:
            self.name = self.data_display.get_member_name(
                self.member,
                self.language
            )

        Selector._ready(self)
    
    def create_entry(self, value, label, selected):

        entry = Element()

        entry.check = CheckBox()
        entry.check["name"] = self.name
        entry.check.value = selected
        entry.check["value"] = value
        entry.append(entry.check)

        entry.label = Element("label")
        entry.label["for"] = entry.check.require_id()
        entry.label.append(label)
        entry.append(entry.label)

        return entry

    def insert_into_form(self, form, field_instance):
        
        field_instance.append(self)

        # Disable the 'required' mark for this field, as it doesn't make sense
        # on a checklist
        required_mark = getattr(field_instance.label, "required_mark", None)

        if required_mark and \
        not (self.member and \
        self.member.min and \
        isinstance(self.member.min, int) and \
        self.member.min > 0):
            required_mark.visible = False




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2007
"""
from itertools import groupby

def filter_by(collection, **kwargs):
    
    for item in collection:
        for key, value in kwargs.iteritems():
            if not getattr(item, key) == value:
                break
        else:
            yield item

def first(collection, **kwargs):
    
    if kwargs:
        collection = filter_by(collection, **kwargs)

    try:
        return iter(collection).next()
    except StopIteration:
        return None

def last(collection, **kwargs):
    
    if kwargs:
        collection = filter_by(collection, **kwargs)

    for item in collection:
        pass

    return item

def is_empty(collection):
    """Indicates if the given iterable object contains at least one item. Note
    that calling this function on an iterator will consume its first item.

    @param collection: The iterable object to be tested.
    @type collection: iterable

    @return: True if the object contains at least one item, False if it is
        empty.
    @rtype: bool
    """
    try:
        iter(collection).next()
    except StopIteration:
        return True
    else:
        return False

def grouped(collection, key):

    if isinstance(key, basestring):
        attrib = key
        key = lambda item: getattr(item, attrib, None)

    return groupby(sorted(collection, key = key), key = key)




#-*- coding: utf-8 -*-
u"""
Utilities for writing application controllers.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from cocktail.controllers.requesthandler import RequestHandler
from cocktail.controllers.controller import Controller
from cocktail.controllers.formcontrollermixin import FormControllerMixin
from cocktail.controllers.dispatcher import (
    Dispatcher,
    StopRequest,
    context
)
from cocktail.controllers.location import Location
from cocktail.controllers.viewstate import (
    get_state,
    view_state,
    view_state_form
)
from cocktail.controllers.parameters import (
    serialize_parameter, get_parameter, FormSchemaReader, CookieParameterSource
)
from cocktail.controllers.usercollection import UserCollection
import cocktail.controllers.grouping
import cocktail.controllers.erroremail
import cocktail.controllers.handlerprofiler




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from types import FunctionType
from threading import local, Lock
from ZODB import DB
import transaction
from cocktail.modeling import getter
from cocktail.events import Event


class DataStore(object):
    """A thread safe wrapper over the application's object database. Normally
    used through its global L{datastore} instance.
    
    The class expects an X{storage} setting, containing a
    L{storage<ZODB.BaseStorage.BaseStorage>} instance pointing to the physical
    location of the database (see the ZODB documentation for more details).
    """
    def __init__(self, storage = None):
        self._thread_data = local()
        self.__storage_lock = Lock()
        self.__db = None
        self.__storage = None
        self.storage = storage
    
    storage_changed = Event("""
        An event triggered when changing which storage is used by the
        datastore.
        """)

    connection_opened = Event("""
        An event triggered when the datastore spawns a new thread-bound
        connection.

        @ivar connection: The new connection.
        @type connection: L{Connection<>}
        """)

    def _get_storage(self):
        if isinstance(self.__storage, FunctionType):            
            self.__storage_lock.acquire()
            try:
                if isinstance(self.__storage, FunctionType):
                    self.__storage = self.__storage()
            finally:
                self.__storage_lock.release()
        return self.__storage

    def _set_storage(self, storage):
        self.__storage_lock.acquire()
        try:
            self.__storage = storage
            self.__db = None
            self.storage_changed()
        finally:
            self.__storage_lock.release()

    storage = property(_get_storage, _set_storage, doc = """
        Gets or sets the underlying ZODB storage for the data store.
        @type: L{ZODB.Storage}
        """)

    @getter
    def db(self):
        if self.__db is None:
            self.__db = DB(self.storage)

        return self.__db

    @getter
    def root(self):
        """Gives access to the root container of the database. The property is
        thread safe; accessing it on different threads will produce different
        containers, each bound to a separate database connection.
        @type: mapping 
        """
        root = getattr(self._thread_data, "root", None)

        if root is None:
            root = self.connection.root()
            self._thread_data.root = root

        return root

    @getter
    def connection(self):
        """Returns the database connection for the current thread. The property
        is thread safe; accessing it on different threads will produce
        different connections. Once called, each connection remains bound to a
        single thread until the thread finishes or the datastore's L{close}
        method is called.
        @type: L{Connection<ZODB.Connection.Connection>}
        """
        connection = getattr(self._thread_data, "connection", None)

        if connection is None:
            connection = self.db.open()
            self._thread_data.connection = connection
            self.connection_opened(connection = connection)

        return connection

    commit = transaction.commit
    abort = transaction.abort

    def close(self):
        """Closes the connection to the database for the current thread.
        Accessing the L{root} or L{connection} properties after this method is
        called will spawn a new database connection.
        """
        if hasattr(self._thread_data, "root"):
            self._thread_data.root = None

        connection = getattr(self._thread_data, "connection", None)

        if connection is not None:
            self._thread_data.connection.close()
            self._thread_data.connection = None

    def sync(self):
        self._thread_data.root = None
        self.connection.sync()

datastore = DataStore()




#-*- coding: utf-8 -*-
"""Apply percent encoding to IRI fragments.

Adapted from a script by Joe Gregorio (joe@bitworking.org) under the MIT
license.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			April 2009
"""

escape_range = [
   (0xA0, 0xD7FF),
   (0xE000, 0xF8FF),
   (0xF900, 0xFDCF),
   (0xFDF0, 0xFFEF),
   (0x10000, 0x1FFFD),
   (0x20000, 0x2FFFD),
   (0x30000, 0x3FFFD),
   (0x40000, 0x4FFFD),
   (0x50000, 0x5FFFD),
   (0x60000, 0x6FFFD),
   (0x70000, 0x7FFFD),
   (0x80000, 0x8FFFD),
   (0x90000, 0x9FFFD),
   (0xA0000, 0xAFFFD),
   (0xB0000, 0xBFFFD),
   (0xC0000, 0xCFFFD),
   (0xD0000, 0xDFFFD),
   (0xE1000, 0xEFFFD),
   (0xF0000, 0xFFFFD),
   (0x100000, 0x10FFFD)
]
 
def percent_encode(c):
    i = ord(c)
    for low, high in escape_range:
        if i < low:
            break
        if i <= high:
            return "".join("%%%2X" % ord(b) for b in c.encode('utf-8'))
    return c




#-*- coding: utf-8 -*-
u"""
This package provides a set of high level interfaces for object persistence.
It's built as a declarative layer over Zope's Object Data Base (ZODB), adding
support for multi-language content and declarative queries over indexes.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from cocktail.persistence.persistentlist import PersistentList
from cocktail.persistence.persistentmapping import PersistentMapping
from cocktail.persistence.persistentset import PersistentSet
from cocktail.persistence.persistentorderedset import PersistentOrderedSet
from cocktail.persistence.persistentrelations import (
    PersistentRelationList,
    PersistentRelationSet,
    PersistentRelationOrderedSet,
    PersistentRelationMapping
)
from cocktail.persistence.persistentobject import (
    PersistentClass, PersistentObject, UniqueValueError, NewObjectDeletedError
)
from cocktail.persistence.datastore import datastore
from cocktail.persistence.incremental_id import incremental_id
from cocktail.persistence.index import Index
from cocktail.persistence import indexing
from cocktail.persistence.query import Query




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2007
"""
import re

XHTML1_STRICT = u"""<!DOCTYPE html
PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">"""

XHTML1_TRANSITIONAL = u"""<!DOCTYPE html
PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">"""

HTML4_STRICT = u"""<!DOCTYPE html
PUBLIC "-//W3C//DTD HTML 4.01//EN"
"http://www.w3.org/TR/html4/strict.dtd">"""

HTML4_TRANSITIONAL = u"""<!DOCTYPE html
PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
"http://www.w3.org/TR/html4/loose.dtd">"""

_entity_expr = re.compile("[\"<>&]")
_entity_dict = {
    "\"": "&quot;",
    "<": "&lt;",
    ">": "&gt;",
    "&": "&amp;"
}
_entity_translation = lambda match: _entity_dict[match.group(0)]


class Renderer(object):

    doctype = None
    single_tags = "img", "link", "meta", "br", "hr", "input"
    flag_attributes = "selected", "checked"

    def __init__(self):
        self.__before_rendering = []
        self.__after_rendering = []

    def make_page(self, element):
        
        from cocktail.html.page import Page

        page = Page()
        page.doctype = self.doctype
        page.body.append(element)

        if element.page_content_type:
            page.content_type = element.page_content_type

        if element.page_charset:
            page.charset = element.page_charset

        return page

    def before_element_rendered(self, handler):
        self.__before_rendering.append(handler)

    def after_element_rendered(self, handler):
        self.__after_rendering.append(handler)

    def write_element(self, element, out):

        for handler in self.__before_rendering:
            handler(element, self)

        if element.client_model:
            out("<script type='text/javascript'>")
            out("cocktail._clientModel('%s').html = '" % element.client_model)
            wrapped_out = out
            def out(snippet):
                wrapped_out(snippet.replace("'", "\\'").replace("\n", "\\n"))

        tag = element.tag
        render_children = True

        if tag:
            # Tag opening
            out(u"<" + tag)

            # Attributes
            for key, value in element.attributes.iteritems():
                if value is not None \
                and not (isinstance(value, bool) and not value):

                    out(u" ")

                    if key in self.flag_attributes:
                        if value:
                            self._write_flag(key, out)
                    else:
                        value = _entity_expr.sub(
                            _entity_translation,
                            unicode(value)
                        )
                        out(key + u'="' + value + u'"')

            # Single tag closure
            if tag in self.single_tags:

                if element.children:
                    raise RenderingError(
                        "Children not allowed on <%s> tags" % tag)

                out(self.single_tag_closure)
                render_children = False

            # Beginning of tag content
            else:
                out(u">")

        if render_children:

            for child in element.children:
                child._render(self, out)

            element._content_ready()

            if tag:
                out(u"</" + tag + u">")

        if element.client_model:
            wrapped_out("';</script>")

        for handler in self.__after_rendering:
            handler(element, self)


class HTML4Renderer(Renderer):

    doctype = HTML4_STRICT
    single_tag_closure = u">"

    def _write_flag(self, key, out):
        out(key)


class XHTMLRenderer(Renderer):

    doctype = XHTML1_STRICT
    single_tag_closure = u"/>"

    def _write_flag(self, key, out):
        out(key + u'="' + key + u'"')


DEFAULT_RENDERER_TYPE = HTML4Renderer

class RenderingError(Exception):
    pass




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
from cocktail.schema import Member
from cocktail.controllers.usercollection import UserCollection


class UserCollectionMember(Member):

    type = UserCollection
    root_type = None
    edit_control = "cocktail.html.CollectionView"

    allow_type_selection = True
    allow_member_selection = True
    allow_language_selection = True
    allow_filters = True
    allow_sorting = True
    allow_grouping = True
    allow_paging = True

    def read_request_value(self, reader):
        
        user_collection = self.type(self.root_type)
        user_collection.params.prefix = self.name
        user_collection.params.source = reader.source
        
        user_collection.allow_type_selection = self.allow_type_selection
        user_collection.allow_member_selection = self.allow_member_selection
        user_collection.allow_language_selection = self.allow_language_selection
        user_collection.allow_filters = self.allow_filters
        user_collection.allow_sorting = self.allow_sorting
        user_collection.allow_grouping = self.allow_grouping
        user_collection.allow_paging = self.allow_paging

        return user_collection




#-*- coding: utf-8 -*-
u"""
Component-based (X)HTML rendering.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			November 2007
"""
from cocktail.html.element import Element, Content, PlaceHolder
from cocktail.html.resources import Resource, Script, StyleSheet
from cocktail.html.overlay import Overlay




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from persistent import Persistent
from cocktail.modeling import empty_set
from cocktail.persistence.persistentset import PersistentSet

infinite = object()


class Index(Persistent):
    """A persistent index that supports multiple entries per key. Used to
    maintain an index for a non unique field.
    """

    def __init__(self, mapping):
        self.__groups = mapping
        self.__none_entries = PersistentSet()

    def add(self, key, value):

        if key is None:
            self.__none_entries.add(value)
            self._p_changed = True
        else:
            group = self.__groups.get(key)

            if group is None:
                self.__groups[key] = group = PersistentSet()
         
            group.add(value)
            
            self._p_changed = True
            self.__groups._p_changed = True

    def remove(self, key, value):
        
        if key is None:
            self.__none_entries.discard(value)
            self._p_changed = True
        else:
            group = self.__groups.get(key)
            
            if group is not None:
                group.discard(value)

                if not group:
                    del self.__groups[key]

                self._p_changed = True
                self.__groups._p_changed = True

    def __getitem__(self, key):
        
        if isinstance(key, slice):        
            raise ValueError(
                "Slicing an index is not supported; use keys()/values() "
                "instead")
        else:
            if key is None:
                return self.__none_entries
            else:
                return self.__groups.get(key, empty_set)

    def __delitem__(self, key):
        if key is None:
            self.__none_entries = PersistentSet()
        else:
            self.__groups.__delitem__(key)
            self.__groups._p_changed = True

        self._p_changed = True

    def keys(self,
        min = infinite,
        max = infinite,
        excludemin = False,
        excludemax = False):
        
        return list(self.iterkeys(
            min = min,
            max = max,
            excludemin = excludemin,
            excludemax = excludemax            
        ))

    def values(self,
        min = infinite,
        max = infinite,
        excludemin = False,
        excludemax = False):
        
        return list(self.itervalues(
            min = min,
            max = max,
            excludemin = excludemin,
            excludemax = excludemax            
        ))

    def items(self,
        min = infinite,
        max = infinite,
        excludemin = False,
        excludemax = False):
        
        return list(self.iteritems(
            min = min,
            max = max,
            excludemin = excludemin,
            excludemax = excludemax            
        ))

    def itervalues(self,
        min = infinite,
        max = infinite,
        excludemin = False,
        excludemax = False):

        if (min is infinite or (min is None and not excludemin)) \
        and (max is not None or (max is None and not excludemax)) \
        and self.__none_entries:
            for item in self.__none_entries:
                yield item
       
        if max is not None:

            if min is infinite:
                min = None

            if max is infinite:
                max = None
            
            for group in self.__groups.itervalues(
                min = min,
                max = max,
                excludemin = excludemin and min is not None,
                excludemax = excludemax and max is not None
            ):
                for item in group:
                    yield item

    def iterkeys(self,
        min = infinite,
        max = infinite,
        excludemin = False,
        excludemax = False):
        
        if (min is infinite or (min is None and not excludemin)) \
        and (max is not None or (max is None and not excludemax)) \
        and self.__none_entries:
            yield None
        
        if max is not None:

            if min is infinite:
                min = None

            if max is infinite:
                max = None
            
            for key in self.__groups.iterkeys(
                min = min,
                max = max,
                excludemin = excludemin and min is not None,
                excludemax = excludemax and max is not None
            ):
                yield key

    def iteritems(self,
        min = infinite,
        max = infinite,
        excludemin = False,
        excludemax = False):
        
        if (min is infinite or (min is None and not excludemin)) \
        and (max is not None or (max is None and not excludemax)) \
        and self.__none_entries:
            for item in self.__none_entries:
                yield None, item

        if max is not None:

            if min is infinite:
                min = None

            if max is infinite:
                max = None

            for key, group in self.__groups.iteritems(
                min = min,
                max = max,
                excludemin = excludemin and min is not None,
                excludemax = excludemax and max is not None
            ):                
                for item in group:
                    yield key, item

    def minKey(self):        
        if self.__none_entries:
            return None
        else:
            return self.__groups.minKey()

    def maxKey(self):
        try:
            return self.__groups.maxKey()
        except ValueError:
            if self.__none_entries:
                return None
            else:
                raise

    def __len__(self):
        count = self.__groups.__len__()

        if self.__none_entries:
            count += 1

        return count

    def __iter__(self):
        if self.__none_entries:
            yield None

        for group in self.__groups:
            yield group

    def __contains__(self, key):
        if key is None:
            return bool(self.__none_entries)
        else:
            return self.__groups.__contains__(key)
    
    def __notzero__(self):
        return self.__none_entries or self.__groups

    def has_key(self, key):
        
        if key is None:
            return bool(self.__none_entries)
        else:
            return self.__groups.has_key(key)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			March 2009
"""
from unittest import TestCase
from cocktail.tests.utils import EventLog


class SchemaEventsTestCase(TestCase):

    def test_member_added_event(self):

        from cocktail.schema import Schema, String
        
        foo = Schema("foo")
        spam = Schema("spam")
        spam.inherit(foo)

        events = EventLog()
        events.listen(foo.member_added, spam.member_added)

        bar = String("bar")
        foo.add_member(bar)

        scrum = String("scrum")
        foo.add_member(scrum)

        event = events.pop(0)
        self.assertEqual(event.slot, foo.member_added)
        self.assertEqual(event.member, bar)

        event = events.pop(0)
        self.assertEqual(event.slot, foo.member_added)
        self.assertEqual(event.member, scrum)

        self.assertFalse(events)

    def test_inherited_event(self):

        from cocktail.schema import Schema

        foo = Schema()
        events = EventLog()
        events.listen(foo_inherited = foo.inherited)

        # Basic inheritance
        bar = Schema()
        bar.inherit(foo)

        event = events.pop(0)
        self.assertEqual(event.slot, foo.inherited)
        self.assertEqual(event.schema, bar)
        
        events.listen(bar_inherited = bar.inherited)

        # Nested inheritance
        spam = Schema()
        spam.inherit(bar)

        event = events.pop(0)
        self.assertEqual(event.slot, foo.inherited)
        self.assertEqual(event.schema, spam)

        event = events.pop(0)
        self.assertEqual(event.slot, bar.inherited)
        self.assertEqual(event.schema, spam)

        # Multiple inheritance
        scrum = Schema()

        events.listen(scrum_inherited = scrum.inherited)

        snutch = Schema()
        snutch.inherit(foo, scrum)
        
        event = events.pop(0)
        self.assertEqual(event.slot, foo.inherited)
        self.assertEqual(event.schema, snutch)

        event = events.pop(0)
        self.assertEqual(event.slot, scrum.inherited)
        self.assertEqual(event.schema, snutch)


class SchemaGroupsTestCase(TestCase):

    def test_grouped_members(self):

        from cocktail.schema import Schema, Member

        a1 = Member("a1", member_group = "a")
        a2 = Member("a2", member_group = "a")
        b1 = Member("b1", member_group = "b")
        b2 = Member("b2", member_group = "b")
        z = Member("z")
        
        schema = Schema(members = [a1, b2, a2, z, b1])
        schema.members_order = ["a2", "a1", "b2", "b1"]
        schema.groups_order = "a", "b"
        
        groups = schema.grouped_members()

        assert len(groups) == 3
        assert all(isinstance(group, tuple) for group in groups)
        assert groups[0][0] == None
        assert groups[1][0] == "a"
        assert groups[2][0] == "b"
        assert list(groups[0][1]) == [z]
        assert list(groups[1][1]) == [a2, a1]
        assert list(groups[2][1]) == [b2, b1]




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2008
"""
import cherrypy
from cgi import parse_qs, escape
from urllib import urlencode

def get_state(**kwargs):
    state = parse_qs(cherrypy.request.query_string)
    state.update(kwargs)
    
    for key, value in state.items():
        if value is None:
            del state[key]

    return state

def view_state(**kwargs):
    return urlencode(get_state(**kwargs), True)

def view_state_form(**kwargs):

    form = []

    for key, values in get_state(**kwargs).iteritems():
        if values is not None:
            for value in values:
                form.append(
                    "<input type='hidden' name='%s' value='%s'>"
                    % (key, escape(value))
                )

    return "\n".join(form)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import re
from datetime import datetime
from decimal import Decimal
from cocktail.pkgutils import get_full_name
from cocktail.translations.translation import (
    translations,
    ca_possessive,
    ca_join,
    es_join,
    en_join,
    plural2
)
from cocktail.schema.expressions import PositiveExpression

translations.define("bool-instance",
    ca = lambda instance: u"Sí" if instance else u"No",
    es = lambda instance: u"Sí" if instance else u"No",
    en = lambda instance: u"Yes" if instance else u"No"
)

translations.define("Any",
    ca = u"Qualsevol",
    es = u"Cualquiera",
    en = u"Any"
)

translations.define("Accept",
    ca = u"Acceptar",
    es = u"Aceptar",
    en = u"Accept"
)

translations.define("Cancel",
    ca = u"Cancel·lar",
    es = u"Cancelar",
    en = u"Cancel"
)

translations.define("Select",
    ca = u"Seleccionar",
    es = u"Seleccionar",
    en = u"Select"
)

translations.define("Submit",
    ca = u"Confirmar",
    es = u"Confirmar",
    en = u"Submit"
)

translations.define("Item count",
    ca = lambda page_range, item_count: \
        u"<strong>%d-%d</strong> de %d" % (
            page_range[0], page_range[1], item_count
        ),
    es = lambda page_range, item_count: \
        u"<strong>%d-%d</strong> de %d" % (
            page_range[0], page_range[1], item_count
        ),
    en = lambda page_range, item_count: \
        u"<strong>%d-%d</strong> of %d" % (
            page_range[0], page_range[1], item_count
        )
)

translations.define("Results per page",
    ca = u"Resultats per pàgina",
    es = u"Resultados por página",
    en = u"Results per page"
)

translations.define("cocktail.html.CollectionView no search results",
    ca = u"La cerca no conté cap element.",
    es = u"La búsqueda no contiene ningún elemento.",
    en = u"The search has no matches."
)

translations.define("cocktail.html.CollectionView no results",
    ca = u"La vista activa no conté cap element.",
    es = u"La vista activa no contiene ningún elemento.",
    en = u"The current view has no matching items."
)

translations.define("Visible members",
    ca = u"Camps",
    es = u"Campos",
    en = u"Fields"
)

translations.define("Visible languages",
    ca = u"Idiomes",
    es = u"Idiomas",
    en = u"Languages"
)

translations.define("Filters",
    ca = u"Filtres",
    es = u"Filtros",
    en = u"Filters"
)

translations.define("Redirecting",
    ca = u"Redirecció",
    es = u"Redirección",
    en = u"Redirecting"
)

translations.define("Redirection explanation",
    ca = u"S'està redirigint la petició. Si no ets automàticament redirigit, "
         u"prem el botó per continuar navegant pel web.",
    es = u"Se está redirigiendo la petición. Si no eres automáticamente "
         u"redirigido pulsa el botón para proseguir con la navegación.",
    en = u"Redirection in process. If you aren't redirected automatically "
         u"press the button to proceed."
)

translations.define("Redirection button",
    ca = u"Continuar",
    es = u"Continuar",
    en = u"Continue"
)

translations.define("cocktail.html.CollectionView search",
    ca = u"Cerca",
    es = u"Búsqueda",
    en = u"Search"
)

translations.define("cocktail.html.ContentView search results",
    ca = u"Resultats de la cerca:",
    es = u"Resultados de la búsqueda:",
    en = u"Search results:"
)

translations.define("cocktail.html.CollectionView selection options",
    ca = u"Seleccionar:",
    es = u"Seleccionar:",
    en = u"Select:"
)

translations.define("cocktail.html.CollectionView select all",
    ca = u"Tots",
    es = u"Todos",
    en = u"All"
)

translations.define("cocktail.html.CollectionView clear selection",
    ca = u"Cap",
    es = u"Ninguno",
    en = u"None"
)

translations.define("cocktail.html.modal_selector accept button",
    ca = u"Acceptar",
    es = u"Aceptar",
    en = u"Accept"
)

translations.define("cocktail.html.modal_selector cancel button",
    ca = u"Cancel·lar",
    es = u"Cancelar",
    en = u"Cancel"
)

translations.define("cocktail.html.modal_selector select button",
    ca = u"Seleccionar",
    es = u"Seleccionar",
    en = u"Select"
)

translations.define("Translations",
    ca = u"Traduccions",
    es = u"Traducciones",
    en = u"Translations"
)

translations.define("date format",
    ca = "%d/%m/%Y",
    es = "%d/%m/%Y",
    en = "%m/%d/%Y"
)

translations.define("jquery_date format",
    ca = "dd/mm/yy",
    es = "dd/mm/yy",
    en = "mm/dd/yy"
)

# -------- ABBREVIATED TRANSLATIONS OF MONTHS
translations.define("month 1 abbr", ca = u"Gen", es = u"Ene", en = u"Jan")
translations.define("month 2 abbr", ca = u"Feb", es = u"Feb", en = u"Feb")
translations.define("month 3 abbr", ca = u"Mar", es = u"Mar", en = u"Mar")
translations.define("month 4 abbr", ca = u"Abr", es = u"Abr", en = u"Apr")
translations.define("month 5 abbr", ca = u"Mai", es = u"May", en = u"May")
translations.define("month 6 abbr", ca = u"Jun", es = u"Jun", en = u"Jun")
translations.define("month 7 abbr", ca = u"Jul", es = u"Jul", en = u"Jul")
translations.define("month 8 abbr", ca = u"Ago", es = u"Ago", en = u"Aug")
translations.define("month 9 abbr", ca = u"Set", es = u"Sep", en = u"Sep")
translations.define("month 10 abbr", ca = u"Oct", es = u"Oct", en = u"Oct")
translations.define("month 11 abbr", ca = u"Nov", es = u"Nov", en = u"Nov")
translations.define("month 12 abbr", ca = u"Des", es = u"Dic", en = u"Dec")

# -------- FULLTEXT TRANSLATIONS OF MONTHS
translations.define("month 1", ca = u"Gener", es = u"Enero", en = u"January")
translations.define("month 2", ca = u"Febrer", es = u"Febrero", en = u"February")
translations.define("month 3", ca = u"Març", es = u"Marzo", en = u"March")
translations.define("month 4", ca = u"Abril", es = u"Abril", en = u"April")
translations.define("month 5", ca = u"Maig", es = u"Mayo", en = u"May")
translations.define("month 6", ca = u"Juny", es = u"Junio", en = u"June")
translations.define("month 7", ca = u"Juliol", es = u"Julio", en = u"July")
translations.define("month 8", ca = u"Agost", es = u"Agosto", en = u"August")
translations.define("month 9", ca = u"Setembre", es = u"Septiembre", en = u"September")
translations.define("month 10", ca = u"Octubre", es = u"Octubre", en = u"October")
translations.define("month 11", ca = u"Novembre", es = u"Noviembre", en = u"November")
translations.define("month 12", ca = u"Desembre", es = u"Diciembre", en = u"December")

# -------- ABBREVIATED TRANSLATIONS OF WEEKDAYS
translations.define("weekday 0 abbr", ca = u"Dl", es = u"Lun", en = u"Mon")
translations.define("weekday 1 abbr", ca = u"Dt", es = u"Mar", en = u"Tue")
translations.define("weekday 2 abbr", ca = u"Dc", es = u"Mié", en = u"Wed")
translations.define("weekday 3 abbr", ca = u"Dj", es = u"Jue", en = u"Thu")
translations.define("weekday 4 abbr", ca = u"Dv", es = u"Vie", en = u"Fri")
translations.define("weekday 5 abbr", ca = u"Ds", es = u"Sáb", en = u"Sat")
translations.define("weekday 6 abbr", ca = u"Dg", es = u"Dom", en = u"Sun")

# -------- FULLTEXT TRANSLATIONS OF WEEKDAYS
translations.define("weekday 0", ca = u"Dilluns", es = u"Lunes", en = u"Monday")
translations.define("weekday 1", ca = u"Dimarts", es = u"Martes", en = u"Tuesday")
translations.define("weekday 2", ca = u"Dimecres", es = u"Miércoles", en = u"Wednesday")
translations.define("weekday 3", ca = u"Dijous", es = u"Jueves", en = u"Thursday")
translations.define("weekday 4", ca = u"Divendres", es = u"Viernes", en = u"Friday")
translations.define("weekday 5", ca = u"Dissabte", es = u"Sábado", en = u"Saturday")
translations.define("weekday 6", ca = u"Diumenge", es = u"Domingo", en = u"Sunday")

DATE_STYLE_NUMBERS = 1
DATE_STYLE_ABBR = 2
DATE_STYLE_TEXT = 3

def _date_instance_ca(instance, style = DATE_STYLE_NUMBERS):
    if style == DATE_STYLE_NUMBERS :
        return instance.strftime(translations("date format", "ca"))
    if style == DATE_STYLE_ABBR :
        return u"%s %s %s" % \
            (instance.day,
            translations(u"month %s %s" % (instance.month,"abbr"), "ca"),
            instance.year)
    if style == DATE_STYLE_TEXT :
        return u"%s %s de %s" % \
            (instance.day,
            ca_possessive(
                translations(u"month %s" % (instance.month), "ca").lower()
            ),
            instance.year)

def _date_instance_es(instance, style = DATE_STYLE_NUMBERS):
    if style == DATE_STYLE_NUMBERS :
        return instance.strftime(translations("date format", "es"))
    if style == DATE_STYLE_ABBR :
        return u"%s %s %s" % \
            (instance.day,
            translations(u"month %s %s" % (instance.month,"abbr"), "es"),
            instance.year)
    if style == DATE_STYLE_TEXT :
        return u"%s de %s de %s" % \
            (instance.day,
            translations(u"month %s" % (instance.month), "es"),
            instance.year)

def _date_instance_en(instance, style = DATE_STYLE_NUMBERS):
    if style == DATE_STYLE_NUMBERS :
        return instance.strftime(translations("date format", "en"))
    if style == DATE_STYLE_ABBR :
        return u"%s %s %s" % \
            (instance.year,
            translations(u"month %s %s" % (instance.month,"abbr"), "en"),
            instance.day)
    if style == DATE_STYLE_TEXT :
        return u"%s %s %s" % \
            (instance.year,
            translations(u"month %s" % (instance.month), "en"),
            instance.day)

translations.define("date-instance",
    ca = _date_instance_ca,
    es = _date_instance_es,
    en = _date_instance_en
)

translations.define("datetime-instance",
    ca = lambda instance, style = DATE_STYLE_NUMBERS:
        _date_instance_ca(instance, style) + instance.strftime(" %H:%M:%S"),
    es = lambda instance, style = DATE_STYLE_NUMBERS:
        _date_instance_es(instance, style) + instance.strftime(" %H:%M:%S"),
    en = lambda instance, style = DATE_STYLE_NUMBERS:
        _date_instance_en(instance, style) + instance.strftime(" %H:%M:%S")
)

def _create_time_span_function(language, span_format, forms, join):

    def time_span(span):

        # Without days
        if len(span) == 4:
            return span_format % span[:-1]
        
        # With days
        else:
            desc = []
            
            for value, form in zip(span, forms):
                if value:
                    desc.append("%d %s" % (value, plural2(value, *form)))

            return join(desc)

    time_span.func_name = language + "_time_span"
    return time_span

translations.define("time span",
    ca = _create_time_span_function(
        "ca",
        "%.2d.%.2d.%2d", 
        [[u"dia", u"dies"],
         [u"hora", u"hores"],
         [u"minut", u"minuts"],
         [u"segon", u"segons"],
         [u"mil·lisegon", u"mil·lisegons"]],
        ca_join
    ),
    es = _create_time_span_function(
        "es",
        "%.2d:%.2d:%2d", 
        [[u"día", u"días"],
         [u"hora", u"horas"],
         [u"minuto", u"minutos"],
         [u"segundo", u"segundos"],
         [u"milisegundo", u"milisegundos"]],
        es_join
    ),
    en = _create_time_span_function(
        "en",
        "%.2d:%.2d:%2d", 
        [[u"day", u"days"],
         [u"hour", u"hours"],
         [u"minute", u"minutes"],
         [u"second", u"seconds"],
         [u"millisecond", u"milliseconds"]],
        en_join
    )
)

# html.FilterBox
#------------------------------------------------------------------------------
translations.define("cocktail.html.FilterBox add filter",
    ca = u"Afegir filtre",
    es = u"Añadir filtro",
    en = u"Add filter"
)

translations.define("cocktail.html.FilterBox remove filters",
    ca = u"Treure filtres",
    es = u"Quitar filtros",
    en = u"Remove filters"
)

translations.define("cocktail.html.FilterBox discard filters",
    ca = u"Descartar",
    es = u"Descartar",
    en = u"Discard"
)

translations.define("cocktail.html.FilterBox apply filters",
    ca = u"Cercar",
    es = u"Buscar",
    en = u"Search"
)

translations.define("cocktail.html.UserFilterEntry operator eq",
    ca = u"Igual a",
    es = u"Igual a",
    en = u"Equals"
)

translations.define("cocktail.html.UserFilterEntry operator ne",
    ca = u"Diferent de",
    es = u"Distinto de",
    en = u"Not equals"
)

translations.define("cocktail.html.UserFilterEntry operator gt",
    ca = u"Major que",
    es = u"Mayor que",
    en = u"Greater than"
)

translations.define("cocktail.html.UserFilterEntry operator ge",
    ca = u"Major o igual que",
    es = u"Mayor o igual que",
    en = u"Greater or equal than"
)

translations.define("cocktail.html.UserFilterEntry operator lt",
    ca = u"Menor que",
    es = u"Menor que",
    en = u"Lower than"
)

translations.define("cocktail.html.UserFilterEntry operator le",
    ca = u"Menor o igual que",
    es = u"Menor o igual que",
    en = u"Lower or equal than"
)

translations.define("cocktail.html.UserFilterEntry operator sr",
    ca = u"Conté",
    es = u"Contiene",
    en = u"Contains"
)

translations.define("cocktail.html.UserFilterEntry operator sw",
    ca = u"Comença per",
    es = u"Empieza por",
    en = u"Starts with"
)

translations.define("cocktail.html.UserFilterEntry operator ew",
    ca = u"Acaba en",
    es = u"Acaba en",
    en = u"Ends with"
)

translations.define("cocktail.html.UserFilterEntry operator re",
    ca = u"Coincideix amb l'expressió regular",
    es = u"Coincide con la expresión regular",
    en = u"Matches regular expression"
)

translations.define("cocktail.html.UserFilterEntry operator cn",
    ca = u"Conté",
    es = u"Contiene",
    en = u"Contains"
)

translations.define("cocktail.html.UserFilterEntry operator nc",
    ca = u"No conté",
    es = u"No contiene",
    en = u"Doesn't contain"
)

translations.define("UserFilter.language",
    ca = u"en",
    es = u"en",
    en = u"in"
)

translations.define("cocktail.html.UserFilterEntry any language",
    ca = u"en qualsevol idioma",
    es = u"en cualquier idioma",
    en = u"in any language"
)

translations.define("cocktail.controllers.userfilter.GlobalSearchFilter-instance",
    ca = u"Conté les paraules",
    es = u"Contiene las palabras",
    en = u"Has the words"
)

translations.define(
    "cocktail.controllers.userfilter.DateTimeRangeFilter-instance",
    ca = u"Rang de dates",
    es = u"Rango de fechas",
    en = u"Date range"
)

translations.define(
    "DateTimeRangeFilter.start_date",
    ca = u"Des de",
    es = u"Desde",
    en = u"From"
)

translations.define(
   "DateTimeRangeFilter.end_date",
    ca = u"Fins a",
    es = u"Hasta",
    en = u"Until"
)

translations.define(
    "cocktail.controllers.userfilter.DescendsFromFilter-instance",
    ca = u"Descendència",
    es = u"Descendencia",
    en = u"Descendancy"
)

translations.define("DescendsFromFilter.include_self",
    ca = u"Arrel inclosa",
    es = u"Raíz incluida",
    en = u"Root included"
)

# Languages
#------------------------------------------------------------------------------
translations.define("ca",
    ca = u"Català",
    es = u"Catalán",
    en = u"Catalan"
)

translations.define("es",
    ca = u"Castellà",
    es = u"Español",
    en = u"Spanish"
)

translations.define("en",
    ca = u"Anglès",
    es = u"Inglés",
    en = u"English"
)

translations.define("fr",
    ca = u"Francès",
    es = u"Francés",
    en = u"French",
    fr = u"Français"
)

translations.define("de",
    ca = u"Alemany",
    es = u"Alemán",
    en = u"German"
)

translations.define("translated into",
    ca = lambda lang: "en " + translations(lang, "ca"),
    es = lambda lang: "en " + translations(lang, "es"),
    en = lambda lang: "in " + translations(lang, "en")
)

translations.define("Exception-instance",
    ca = lambda instance: u"Error inesperat (%s: %s)" %
        (instance.__class__.__name__, instance),
    es = lambda instance: u"Error inesperado (%s: %s)" %
        (instance.__class__.__name__, instance),
    en = lambda instance: u"Unexpected error (%s: %s)" %
        (instance.__class__.__name__, instance)
)

# Validation errors
#------------------------------------------------------------------------------
def member_identifier(error, language):
    
    from cocktail.schema import RelationMember

    desc = [
        translations(member, language).lower()
        for member, validable in error.path
        if isinstance(member, RelationMember)
    ]            
    
    if error.language:
        desc.append("%s (%s)" % (
            translations(error.member, language).lower(),
            translations(error.language, language)
        ))
    else:
        desc.append(translations(error.member, language).lower())

    return u" &gt; ".join(desc)

translations.define("cocktail.schema.exceptions.ValidationError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> no és vàlid"
        % member_identifier(instance, "ca"),
    es = lambda instance:
        u"El campo <em>%s</em> no es válido"        
        % member_identifier(instance, "es"),
    en = lambda instance:
        u"The <em>%s</em> field is not valid"
        % member_identifier(instance, "en")
)

translations.define("cocktail.schema.exceptions.ValueRequiredError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> no pot estar buit"
        % member_identifier(instance, "ca"),
    es = lambda instance:
        u"El campo <em>%s</em> no puede estar vacío"
        % member_identifier(instance, "es"),
    en = lambda instance:
        u"The <em>%s</em> field can't be empty"
        % member_identifier(instance, "en")
)

translations.define("cocktail.schema.exceptions.NoneRequiredError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> ha d'estar buit"
        % member_identifier(instance, "ca"),
    es = lambda instance:
        u"El campo <em>%s</em> debe estar vacío"
        % member_identifier(instance, "es"),
    en = lambda instance:
        u"The <em>%s</em> field must be empty"
        % member_identifier(instance, "en")
)

translations.define("cocktail.schema.exceptions.TypeCheckError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> té un tipus incorrecte"
        % member_identifier(instance, "ca"),
    es = lambda instance:
        u"El campo <em>%s</em> tiene un tipo incorrecto"
        % member_identifier(instance, "es"),
    en = lambda instance:
        u"The <em>%s</em> field is of the wrong type"
        % member_identifier(instance, "en")
)

translations.define("cocktail.schema.exceptions.EnumerationError-instance",
    ca = lambda instance:
        u"El valor del camp <em>%s</em> no és un dels permesos"
        % member_identifier(instance, "ca"),
    es = lambda instance:
        u"El campo <em>%s</em> no es uno de los permitidos"
        % member_identifier(instance, "es"),
    en = lambda instance:
        u"Wrong choice on field <em>%s</em>"
        % member_identifier(instance, "en")
)

translations.define("cocktail.schema.exceptions.MinLengthError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> ha de tenir un mínim de %d caràcters"
        % (member_identifier(instance, "ca"), instance.min),
    es = lambda instance:
        u"El campo <em>%s</em> debe tener un mínimo de %d caracteres"
        % (member_identifier(instance, "es"), instance.min),
    en = lambda instance:
        u"The <em>%s</em> field must be at least %d characters long"
        % (member_identifier(instance, "en"), instance.min)
)

translations.define("cocktail.schema.exceptions.MaxLengthError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> ha de tenir un màxim de %d caràcters"
        % (member_identifier(instance, "ca"), instance.max),
    es = lambda instance:
        u"El campo <em>%s</em> debe tener un máximo de %d caracteres"
        % (member_identifier(instance, "es"), instance.max),
    en = lambda instance:
        u"The <em>%s</em> field can't be more than %d characters long"
        % (member_identifier(instance, "en"), instance.max)
)

translations.define("cocktail.schema.exceptions.FormatError-instance",
    ca = lambda instance:
        u"El format del camp <em>%s</em> és incorrecte"
        % member_identifier(instance, "ca"),
    es = lambda instance:
        u"El formato del campo <em>%s</em> es incorrecto"
        % member_identifier(instance, "es"),
    en = lambda instance:
        u"The <em>%s</em> field has a wrong format"
        % member_identifier(instance, "en")
)

translations.define("cocktail.schema.exceptions.MinValueError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> ha de ser igual o superior a %s"
        % (member_identifier(instance, "ca"), instance.min),
    es = lambda instance:
        u"El campo <em>%s</em> debe ser igual o superior a %s"
        % (member_identifier(instance, "es"), instance.min),
    en = lambda instance:
        u"The <em>%s</em> field must be greater or equal than %s"
        % (member_identifier(instance, "en"), instance.min)
)

translations.define("cocktail.schema.exceptions.MaxValueError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> ha de ser igual o inferior a %s"
        % (member_identifier(instance, "ca"), instance.max),
    es = lambda instance:
        u"El campo <em>%s</em> debe ser igual o inferior a %s"
        % (member_identifier(instance, "es"), instance.max),
    en = lambda instance:
        u"The <em>%s</em> field must be lower or equal than %s"
        % (member_identifier(instance, "en"), instance.max)
)

translations.define("cocktail.schema.exceptions.MinItemsError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> ha de contenir %d o més elements"
        % (member_identifier(instance, "ca"), instance.min),
    es = lambda instance:
        u"El campo <em>%s</em> debe contener %d o más elementos"
        % (member_identifier(instance, "es"), instance.min),
    en = lambda instance:
        u"The <em>%s</em> field must contain %d or more items"
        % (member_identifier(instance, "en"), instance.min)
)

translations.define("cocktail.schema.exceptions.MaxItemsError-instance",
    ca = lambda instance:
        u"El camp <em>%s</em> no pot contenir més de %d elements"
        % (member_identifier(instance, "ca"), instance.max),
    es = lambda instance:
        u"El campo <em>%s</em> no puede contener más de %d elementos"
        % (member_identifier(instance, "es"), instance.max),
    en = lambda instance:
        u"The <em>%s</em> field can't contain more than %d items"
        % (member_identifier(instance, "en"), instance.max)
)

translations.define(
    "cocktail.persistence.persistentobject.UniqueValueError-instance",
    ca = lambda instance:
        u"El valor indicat pel camp <em>%s</em> ja existeix a la base de dades"
        % member_identifier(instance, "ca"),
    es = lambda instance:
        u"El valor indicado para el campo <em>%s</em> ya existe en la base de "
        u"datos"
        % member_identifier(instance, "es"),
    en = lambda instance:
        u"The value of the <em>%s</em> field is already in use"
        % member_identifier(instance, "en")
)

translations.define(
    "cocktail.unexpected_error",
    ca = lambda error:
        u"Error inesperat %s" 
        %(error),
    es = lambda error:
        u"Error inesperado %s" 
        %(error),
    en = lambda error:
        u"Unexpedted error %s"
        %(error)
)

# Value parsing
#------------------------------------------------------------------------------

def _thousands_parser(thousands_sep, fraction_sep):

    expr = re.compile(r"^[-+]?((\d{1,3}(\%s\d{3})*)|\d+)(\%s\d+)?$"
                    % (thousands_sep, fraction_sep))
    
    def parser(value):

        if not expr.match(value):
            raise ValueError("Invalid decimal literal: %s" % value)
        
        value = value.replace(thousands_sep, "")

        if fraction_sep != ".":
            value = value.replace(fraction_sep, ".")

        return Decimal(value)

    return parser

def _serialize_thousands(value, thousands_sep, fraction_sep):    
    
    sign, num, precision = value.as_tuple()    
    num = list(num)    
    pos = len(num) + precision
    integer = num[:pos]
    fraction = num[pos:]

    blocks = []

    while integer:
        blocks.insert(0, "".join(str(i) for i in integer[-3:]))
        integer = integer[:-3]

    if blocks:
        serialized_value = thousands_sep.join(blocks)
    else:
        serialized_value = "0"

    if fraction:
        serialized_value += fraction_sep \
                          + str("".join(str(i) for i in fraction))

    if sign:
        serialized_value = "-" + serialized_value

    return serialized_value
    
translations.define("Decimal parser",
    ca = lambda: _thousands_parser(".", ","),
    es = lambda: _thousands_parser(".", ","),
    en = lambda: _thousands_parser(",", ".")
)

translations.define("decimal.Decimal-instance",
    ca = lambda instance: _serialize_thousands(instance, ".", ","),
    es = lambda instance: _serialize_thousands(instance, ".", ","),
    en = lambda instance: _serialize_thousands(instance, ",", ".")
)

# Grouping
#------------------------------------------------------------------------------
translations.define("cocktail.controllers.grouping.MemberGrouping-ascending",
    ca = u"Ascendent",
    es = u"Ascendiente",
    en = u"Ascending"
)

translations.define("cocktail.controllers.grouping.MemberGrouping-descending",
    ca = u"Descendent",
    es = u"Descendiente",
    en = u"Descending"
)

translations.define(
    "cocktail.controllers.grouping.MemberGrouping default variant",
    ca = u"Per valor",
    es = u"Por valor",
    en = u"By value"
)

translations.define("cocktail.controllers.grouping.DateGrouping year variant",
    ca = u"Per any",
    es = u"Por año",
    en = u"By year"
)

translations.define("cocktail.controllers.grouping.DateGrouping month variant",
    ca = u"Per mes",
    es = u"Por mes",
    en = u"By month"
)

translations.define("cocktail.controllers.grouping.DateGrouping day variant",
    ca = u"Per dia",
    es = u"Por día",
    en = u"By day"
)

translations.define("cocktail.controllers.grouping.TimeGrouping hour variant",
    ca = u"Per hora",
    es = u"Por hora",
    en = u"By hour"
)

def _DateGrouping_value(grouping, value, language):
    if grouping.variant == "year":
        return value.year
    elif grouping.variant == "month":
        return "%s %d" % (translations("month %d" % value.month, language), value.year)
    else:
        return translations(value, language, style = DATE_STYLE_TEXT)

translations.define("cocktail.controllers.grouping.DateGrouping value",
    ca = lambda grouping, value: _DateGrouping_value(grouping, value, "ca"),
    es = lambda grouping, value: _DateGrouping_value(grouping, value, "es"),
    en = lambda grouping, value: _DateGrouping_value(grouping, value, "en")
)

# Date interval
#------------------------------------------------------------------------------
def _date_interval_ca(dates = None):
    start_date, end_date = dates

    date_string = ""

    def month_string(date, show = False):
        if date.month in (4, 8, 10):
            return  u"d'%s" % translations(u"month %s" % (date.month), "ca") \
                if show else ""
        else:
            return  u"de %s" % translations(u"month %s" % (date.month), "ca") \
                if show else ""

    def year_string(date, show = False):
        return " de %d" % (date.year,) if show else ""

    show_start_year = show_end_year = show_start_month = show_end_month = False

    if start_date.year != end_date.year:
        show_start_year = show_end_year = show_start_month = show_end_month = True
    elif start_date.month != end_date.month:
        show_end_year = end_date.year != datetime.now().year
        show_start_month = show_end_month = True
    else:
        show_end_year = end_date.year != datetime.now().year
        show_end_month = True

    # One day interval
    if start_date.day == end_date.day and \
        start_date.month == end_date.month and \
        start_date.year == end_date.year:

        date_string = "%d %s%s" % (
            start_date.day, 
            month_string(start_date, True),
            year_string(
                start_date, 
                show = start_date.year != datetime.now().year
            )
        )

    else:
        if start_date.day == 1:
            day_format = u"De l'%d "
        else:
            day_format = u"Del %d "

        date_string = day_format % (start_date.day,)

        if start_date.month == end_date.month \
            and start_date.year == end_date.year:

            date_string += "al %d %s%s" % (
                end_date.day, 
                month_string(end_date, show_end_month),
                year_string(end_date, show_end_year)
            )

        else:
            date_string += "%s%s al %d %s%s" % (
                month_string(start_date, show_start_month),
                year_string(start_date, show_start_year),
                end_date.day,
                month_string(end_date, show_end_month),
                year_string(end_date, show_end_year)
            )

    return date_string

def _date_interval_es(dates = None):
    start_date, end_date = dates

    date_string = ""

    def month_string(date, show = False):
        return  u"de %s" % translations(u"month %s" % (date.month), "es") \
            if show else ""

    def year_string(date, show = False):
        return " de %d" % (date.year,) if show else ""

    show_start_year = show_end_year = show_start_month = show_end_month = False

    if start_date.year != end_date.year:
        show_start_year = show_end_year = show_start_month = show_end_month = True
    elif start_date.month != end_date.month:
        show_end_year = end_date.year != datetime.now().year
        show_start_month = show_end_month = True
    else:
        show_end_year = end_date.year != datetime.now().year
        show_end_month = True

    # One day interval
    if start_date.day == end_date.day and \
        start_date.month == end_date.month and \
        start_date.year == end_date.year:

        date_string = "%d %s%s" % (
            start_date.day, 
            month_string(start_date, True),
            year_string(
                start_date, 
                show = start_date.year != datetime.now().year
            )
        )

    else:
        date_string = u"Del %d " % (start_date.day,)

        if start_date.month == end_date.month \
            and start_date.year == end_date.year:

            date_string += "al %d %s%s" % (
                end_date.day, 
                month_string(end_date, show_end_month),
                year_string(end_date, show_end_year)
            )

        else:
            date_string += "%s%s al %d %s %s" % (
                month_string(start_date, show_start_month),
                year_string(start_date, show_start_year),
                end_date.day,
                month_string(end_date, show_end_month),
                year_string(end_date, show_end_year)
            )

    return date_string

def _date_interval_en(dates = None):
    start_date, end_date = dates

    date_string = ""

    def month_string(date, show = False):
        return  u"%s" % translations(u"month %s" % (date.month), "en") \
            if show else ""

    def year_string(date, show = False):
        return ", %d" % (date.year,) if show else ""

    show_start_year = show_end_year = show_start_month = show_end_month = False

    if start_date.year != end_date.year:
        show_start_year = show_end_year = show_start_month = show_end_month = True
    elif start_date.month != end_date.month:
        show_end_year = end_date.year != datetime.now().year
        show_start_month = show_end_month = True
    else:
        show_end_year = end_date.year != datetime.now().year
        show_end_month = True

    # One day interval
    if start_date.day == end_date.day and \
        start_date.month == end_date.month and \
        start_date.year == end_date.year:

        date_string = "%d %s%s" % (
            start_date.day, 
            month_string(start_date, True),
            year_string(
                start_date, 
                show = start_date.year != datetime.now().year
            )
        )

    else:
        if start_date.month == end_date.month \
            and start_date.year == end_date.year:

            date_string += "From %s %d to %d%s" % (
                month_string(end_date, show_end_month),
                start_date.day, 
                end_date.day, 
                year_string(end_date, show_end_year)
            )

        else:
            date_string += "From %s %d%s to %s %d%s" % (
                month_string(start_date, show_start_month),
                start_date.day,
                year_string(start_date, show_start_year),
                month_string(end_date, show_end_month),
                end_date.day,
                year_string(end_date, show_end_year)
            )

    return date_string

translations.define("Date interval",
    ca = _date_interval_ca,
    es = _date_interval_es,
    en = _date_interval_en
)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from weakref import ref, WeakKeyDictionary
from cocktail.modeling import SynchronizedList
from threading import Lock


def when(event):
    """A decorator factory that attaches decorated functions as event handlers
    for the given event slot.
    
    @param event: The event slot to register the decorated function on.
    @type event: L{EventSlot}

    @return: The decorator that does the binding.
    @rtype: callable
    """
    def decorator(function):
        event.append(function)
        return function

    return decorator


class EventHub(type):
    """A convenience metaclass that automatically registers methods marked with
    the L{event_handler} decorator as class-level event-handlers. 
    """

    def __init__(cls, name, bases, members):
        type.__init__(cls, name, bases, members)

        for key, member in members.iteritems():
            
            if isinstance(member, event_handler):
                handler = getattr(cls, key)
                event_name = key[7:]
                event_slot = getattr(cls, event_name, None)

                if event_slot is None or not isinstance(event_slot, EventSlot):
                    raise TypeError(
                        "Can't attach %s to the %s event on %s, "
                        "indicated event doesn't exist"
                        % (member, event_name, cls)
                    )

                event_slot.append(handler)


class event_handler(classmethod):
    """A decorator that works hand in hand with L{EventHub} in order to ease
    the attachment of handlers to events.
    """


class Event(object):
    """An event describes a certain condition that can be triggered on a class
    and/or its instances. Users of the class can register callback functions
    that will be invoked when the event is triggered, which results in a
    flexible extensibility mechanism.

    This class is a X{descriptor}, and does nothing but produce and cache
    L{event slots<EventSlot>} on the class, its subclasses or its instances, in
    a thread-safe manner. Each element that the event is requested on spawns a
    new slot, which will remain assigned to that element throughout the
    element's life cycle.
    """

    def __init__(self, doc = None):
        self.__doc__ = doc
        self.__slots = WeakKeyDictionary()
        self.__lock = Lock()

    def __get__(self, instance, type = None):
        
        self.__lock.acquire()
                
        try:
            if instance is None:
                return self.__get_slot(type)
            else:
                return self.__get_slot(instance, True)
        finally:
            self.__lock.release()

    def __get_slot(self, target, is_instance = False):

        slot = self.__slots.get(target)

        if slot is None:
            slot = EventSlot()
            slot.event = self
            slot.target = ref(target)
            self.__slots[target] = slot

            if is_instance:
                slot.next = self.__get_slot(target.__class__),
            else:
                slot.next = map(self.__get_slot, target.__bases__)

        return slot


class EventSlot(SynchronizedList):
    """An event slot represents a binding between an L{event<Event>} and one of
    its targets. Slots work as callable lists of callbacks. They possess all
    the usual list methods, plus the capability of being called. When called,
    callbacks are executed in order, receiving any extra parameters through an
    L{EventInfo} instance. The return value of executed callbacks is ignored.

    @ivar event: The event that the slot stores responses for.
    @type event: L{Event}

    @ivar target: The object that the slot is bound to.
    
    @ivar next: A list of other event slots that will be chained up to the
        slot. After a slot has been triggered, and its callbacks processed, its
        chained slots will also be triggered. This is done recursively. For
        example, this is used to trigger class-wide events after activating an
        instance slot.
    """
    event = None
    target = None
    next = ()

    def __call__(self, _event_info = None, **kwargs):
        
        target = self.target()

        # self.target is a weakref, so the object may have expired
        if target is None:
            return
       
        if _event_info is None:
            event_info = EventInfo(kwargs)
            event_info.source = target
        else:
            event_info = _event_info

        event_info.slot = self
        event_info.target = target
        event_info.consumed = False
        
        for callback in self:
            callback(event_info)

            if event_info.consumed:
                break

        for next_slot in self.next:
            next_slot(_event_info = event_info)

        return event_info


class EventInfo(object):
    """An object that encapsulates all the information passed on to callbacks
    when an L{event slot<EventSlot>} is triggered.

    @param target: The element that the event is being triggered on.
    
    @param source: The element that the event originated in. While the
        L{target} changes as the event passes from slot to slot (ie. when
        ascending along derived class), X{source} will always point to the
        first element on which the event was invoked.

    @param slot: The event slot from which the callback is being triggered.
    @type slot: L{EventSlot}

    @param consumed: A flag that allows a callback to prevent the execution of
        any further callback on the slot.
    @type consumed: bool
    """    
    target = None
    source = None
    slot = None
    consumed = False
    
    def __init__(self, params):
        for key, value in params.iteritems():
            setattr(self, key, value)




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2008
"""
from simplejson import dumps
from cocktail.html.element import (
    Element, Content, _thread_data, IdGenerationError
)
from cocktail.html.resources import Resource, Script, StyleSheet
from cocktail.translations import get_language, translations


class Page(Element):

    doctype = None
    content_type = "text/html"
    charset = "utf-8"
    styled_class = False
    tag = "html"

    CORE_SCRIPT = "/cocktail/scripts/core.js"
    JQUERY_SCRIPT = "/cocktail/scripts/jquery.js"

    HTTP_EQUIV_KEYS = frozenset((
        "content-type",
        "expires",
        "refresh",
        "set-cookie"
    ))

    def __init__(self, *args, **kwargs):
        Element.__init__(self, *args, **kwargs)
        self.__element_id = 0
        self.__title = None
        self.__meta = {}
        self.__resource_uris = set()        
        self.__client_translations = set()
        self.__client_variables = {}
        self.__client_params_script = None
        self.__client_model_stack = []

    def _build(self):
      
        self.head = Element("head")
        self.append(self.head)

        self._generated_head = Element()
        self._generated_head.tag = None
        self.head.append(self._generated_head)

        self.body = Element("body")
        self.append(self.body)

        self._body_markup = Content()
        self.append(self._body_markup)

    def _render(self, renderer, out):
        
        if self.doctype:
            out(self.doctype)
            out("\n")           

        # Render the <body> element first, to gather meta information from all
        # elements (meta tags, scripts, style sheets, the page title, etc)
        renderer.before_element_rendered(self._before_descendant_rendered)
        renderer.after_element_rendered(self._after_descendant_rendered)
        
        self._body_markup.value = self.body.render(renderer)
        self._fill_head()
        
        # Then proceed with the full page. The <body> element is hidden so that
        # it won't be rendered a second time
        self.body.visible = False
        Element._render(self, renderer, out)
        self.body.visible = True

    def _before_descendant_rendered(self, descendant, renderer):

        if descendant.client_model:
            self.__client_model_stack.append(descendant.client_model)

        for element in descendant.head_elements:
            self.head.append(Content(element.render(renderer)))

        client_params = descendant.client_params
        client_code = descendant.client_code

        if client_params or client_code:
            
            self._add_resource_to_head(self.JQUERY_SCRIPT)
            self._add_resource_to_head(self.CORE_SCRIPT)
             
            if self.__client_params_script is None:
                
                script_tag = Element("script")
                script_tag["type"] = "text/javascript"
                script_tag.append("jQuery(function () {\n")
                
                self.__client_params_script = Content()
                self.__client_params_script.value = ""
                script_tag.append(self.__client_params_script)
                script_tag.append("});\n")
                self.head.append(script_tag)

            client_model = (
                self.__client_model_stack
                and self.__client_model_stack[-1]
            )

            js = []

            if client_model:
                
                client_model_ref = "\tcocktail._clientModel('%s'" \
                    % client_model

                if not descendant.client_model:
                    client_model_ref += ", '%s'" % descendant["id"]

                client_model_ref += ")"

                if client_params:
                    js.append(client_model_ref + ".params = %s;\n"
                        % dumps(descendant.client_params)
                    )

                if client_code:
                    js.append(client_model_ref + ".code = %s;\n"
                        % dumps(list(descendant.client_code))
                    )
            else:
                js.append(
                    "\tjQuery('#%s').each(function () {" % descendant["id"]
                )
                
                for key, value in client_params.iteritems():
                    js.append("\t\tthis.%s = %s;" % (key, dumps(value)))
                
                js.extend("\t\t" + snippet for snippet in client_code)
                js.append("\t});\n")
            
            self.__client_params_script.value += "\n".join(js)

    def _after_descendant_rendered(self, descendant, renderer):

        page_title = descendant.page_title
        
        # Page title
        if page_title is not None:
            self.__title = page_title

        # Meta tags
        self.__meta.update(descendant.meta)

        # Resources
        for resource in descendant.resources:
            self._add_resource_to_head(resource)
        
        # Client variables and translations
        self.__client_variables.update(descendant.client_variables)
        self.__client_translations.update(descendant.client_translations)

        if descendant.client_model:
            self.__client_model_stack.pop(-1)

    def _fill_head(self):

        head = self._generated_head

        if self.content_type:
            ctype = self.content_type
            if self.charset:
                ctype += ";charset=" + self.charset

            self.__meta["content-type"] = ctype

        for key, value in self.__meta.iteritems():
            key_attrib = key in self.HTTP_EQUIV_KEYS and "http-equiv" or "name"
            meta_tag = Element("meta")
            meta_tag[key_attrib] = key
            meta_tag["content"] = value
            head.append(meta_tag)

        if self.__title and self.__title is not None:
            import re
            html_expr = re.compile(r"</?[^>]+>")
            title_tag = Element("title")
            title_tag.append(html_expr.sub("", self.__title))
            head.append(title_tag)
                        
        if self.__client_translations:
            self._add_resource_to_head(self.CORE_SCRIPT)
            
            language = get_language()
            
            script_tag = Element("script")
            script_tag["type"] = "text/javascript"
            script_tag.append("cocktail.setLanguage(%s);\n" % dumps(language))  
            script_tag.append("\n".join(
                "cocktail.setTranslation(%s, %s);"
                    % (dumps(key), dumps(translations[language][key]))
                for key in self.__client_translations
            ))
            head.append(script_tag)
        
        if self.__client_variables:
            script_tag = Element("script")
            script_tag["type"] = "text/javascript"
            script_tag.append("\n".join(
                "%s = %s;" % (key, dumps(value))
                for key, value in self.__client_variables.iteritems()
            ))
            head.append(script_tag)
        
        if self.CORE_SCRIPT in self.__resource_uris:
            script_tag = Element("script")
            script_tag["type"] = "text/javascript"
            script_tag.append("jQuery(function () { cocktail.init(); });")
            self.head.append(script_tag)

    def _add_resource_to_head(self, resource):
        
        is_string = isinstance(resource, basestring)

        uri = resource if is_string else resource.uri
        
        if uri not in self.__resource_uris:
            self.__resource_uris.add(uri)
            
            if is_string:
                resource = Resource.from_uri(uri)

            if isinstance(resource, Script):
                
                if uri == self.CORE_SCRIPT:
                    self._add_resource_to_head(self.JQUERY_SCRIPT)
                elif uri != self.JQUERY_SCRIPT:
                    self._add_resource_to_head(self.CORE_SCRIPT)

                script_tag = Element("script")
                script_tag["type"] = resource.mime_type
                script_tag["src"] = uri
                self._generated_head.append(script_tag)
    
            elif isinstance(resource, StyleSheet):
                link_tag = Element("link")
                link_tag["rel"] = "Stylesheet"
                link_tag["type"] = resource.mime_type
                link_tag["href"] = uri
                self._generated_head.append(link_tag)
      

